## @file
# GTI Filtering based on S/N

#========================================================================
sub PG_gti_filter(){
    print "\n----> Running PG GTI Filtering for $_[0] .....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Running PG GTI Filtering for $_[0] ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for PG GTI Filtering ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for PG GTI Filtering ..... ";}

    use vars qw($intrument
		$event_list
		$bkg_event_list
		$eregionanal_file
		$image_file
		$lc_src_outfile
		$lc_bkg_outfile 
		$lc_src_counts_file
		$lc_bkg_counts_file
		$pnidl_scriptfile_name
		$m1idl_scriptfile_name
		$m2idl_scriptfile_name
		 );

#.. Check if the user has defined its own coordinates
	&setcoordinates();	
	
    my $source_obsid = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $src_ra       = $gv->SRC_RA;
    my $src_dec      = $gv->SRC_DEC;

    $instrument         = $_[0];
    $bkg_event_list     = $gti_directory."PG_bkg_event_list.fits";
    $src_regionset      = $gti_directory."PG_src_region.ds";
    $bkg_regionset      = $gti_directory."PG_bkg_region.ds";   
    $src_list           = $images_directory.$instrument."_emllist.fits";
    $image_file         = $gti_directory."PG_".$instrument."_img_raw.img";
    $eregionanal_file   = $spectra_directory.$source_name."_".$instrument."_eregionanalyse.txt";
    $lc_src_outfile     = $gti_directory."PG_".$instrument."_lightcurve_src_newgti.fit";
    $lc_bkg_outfile     = $gti_directory."PG_".$instrument."_lightcurve_bkg_newgti.fit";
    $lc_src_counts_file = $gti_directory."PG_".$instrument."_lc_counts_src.txt";
    $lc_bkg_counts_file = $gti_directory."PG_".$instrument."_lc_counts_bkg.txt";

    if ($instrument eq "pn"){$event_list = $pn_event_list;}
    if ($instrument eq "mos1"){$event_list = $m1_event_list;}
    if ($instrument eq "mos2"){$event_list = $m2_event_list;}

    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

#.. Get Starting and Ending Time of the Observation

    my $mode    = $obsid_instruments{$source_obsid}{$instrument}{'mode'};
    my $submode = $obsid_instruments{$source_obsid}{$instrument}{'submode'};
    my $tstart  = $obsid_instruments{$source_obsid}{$instrument}{'starting_time'}; 
    my $tend    = $obsid_instruments{$source_obsid}{$instrument}{'ending_time'};

#.. Generating the ligth curve off the source and the background ...'

#.. Produce Instrument Image (needed by eregioanalyse)

    &produce_image($event_list,$image_file,$instrument);
    
    #.. Using the instrument image to work out the source centroid
    
   	my($bore_img_x,$bore_img_y
     	,$bore_optrad_asec,$bore_optradius
      	,$bore_img_xbkg,$bore_img_ybkg
      	,$bkg_inner_rad_asec,$bkg_inner_rad
      	,$bkg_outer_rad_asec,$bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);	
   	($bore_img_x,$bore_img_y
    	,$bore_optrad_asec,$bore_optradius 
    	,$bore_img_xbkg,$bore_img_ybkg
    	,$bkg_inner_rad_asec,$bkg_inner_rad
    	,$bkg_outer_rad_asec,$bkg_outer_rad)
   		=&run_eregionanalyse($image_file,$src_ra,$src_dec,$instrument,$submode,$eregionanal_file,"PG");
    

#.. Something has gone wrong so wont produce anything

    if ($bore_optrad_asec == 300){                          # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.
	my $rate = 1000.;                                       # set to silly default value
	if ($instrument eq "pn"){$gv->GTI_PN_Rate($rate);}      # Set the PN rate 
	if ($instrument eq "mos1"){$gv->GTI_MOS1_Rate($rate);}  # Set the MOS1 rate       
	if ($instrument eq "mos2"){$gv->GTI_MOS2_Rate($rate);}  # Set the MOS2 rate  
	&error_code(10); 
	return(0);
    }

#.. Set the values of the PG_ parameters: source/background center and radii set to the values found with eregionanalyse

    if ($gv->PG_timebinning == 0){$gv->PG_timebinning(100);} # secs (default value)
    if ($gv->PG_timestart == 0){$gv->PG_timestart($tstart);} # secs
    if ($gv->PG_timeend == 0){$gv->PG_timeend($tend);}       # secs
    if ($gv->PG_minpi == 0){$gv->PG_minpi(200);}             # eV (default value)
    if ($gv->PG_maxpi == 0){$gv->PG_maxpi(10000);}           # eV (default value)
    if ($gv->PG_aringcx == 0){$gv->PG_aringcx($bore_img_xbkg);}    
    if ($gv->PG_aringcy == 0){$gv->PG_aringcy($bore_img_ybkg);} 
    if ($gv->PG_innerrad == 0){$gv->PG_innerrad($bkg_inner_rad);}
    if ($gv->PG_outerrad == 0){$gv->PG_outerrad($bkg_outer_rad);}
    if ($gv->PG_sx == 0){$gv->PG_sx($bore_img_x);}
    if ($gv->PG_sy == 0){$gv->PG_sy($bore_img_y);}
    if ($gv->PG_source_rad == 0){$gv->PG_source_rad($bore_optradius);}

    $src_area = $pi*$gv->PG_source_rad*$gv->PG_source_rad;
    $bkg_area = $pi*$gv->PG_outerrad*$gv->PG_outerrad
	- $pi*$gv->PG_innerrad*$gv->PG_innerrad;
    if ($bkg_area != 0) {$gv->PG_areafactor($src_area/$bkg_area);} else {$gv->PG_areafactor(0.);}

    print "   \#> Source and Background Extraction Regions to be used: \n";
    print "       $instrument \n";
    print "         $instrument Region SRC : ",$gv->PG_src_region,"\n";
    print "         $instrument SRC XC     : ",$gv->PG_sx,"\n";
    print "         $instrument SRC YC     : ",$gv->PG_sy,"\n";
    print "         $instrument SRC RC     : ",$gv->PG_source_rad,"\n";
    print "         $instrument Region BKG : ",$gv->PG_bkg_region, "\n";
    print "         $instrument BKG XC     : ",$gv->PG_aringcx,"\n";
    print "         $instrument BKG YC     : ",$gv->PG_aringcy,"\n";
    print "         $instrument BKG RC     : ",$gv->PG_innerrad," - ",$gv->PG_outerrad,"\n";
    print "         $instrument BKG AREA SCALE FACTOR : ",$gv->PG_areafactor," (Only area taken into account)\n";
    if ($gv->PG_outerrad == 6000){print "         WARNING : $instrument regions : Maximum allowed radius reached";}
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> Source and Background Extraction Regions to be used: \n";
	print MYOUTFILE "       $instrument \n";
	print MYOUTFILE "         $instrument Region SRC : ",$gv->PG_src_region,"\n";
	print MYOUTFILE "         $instrument SRC XC     : ",$gv->PG_sx,"\n";
	print MYOUTFILE "         $instrument SRC YC     : ",$gv->PG_sy,"\n";
	print MYOUTFILE "         $instrument SRC RC     : ",$gv->PG_source_rad,"\n";
	print MYOUTFILE "         $instrument Region BKG : ",$gv->PG_bkg_region,"\n";
	print MYOUTFILE "         $instrument BKG XC     : ",$gv->PG_aringcx,"\n";
	print MYOUTFILE "         $instrument BKG YC     : ",$gv->PG_aringcy,"\n";
	print MYOUTFILE "         $instrument BKG RC     : ",$gv->PG_innerrad," - ",$gv->PG_outerrad,"\n";
	print MYOUTFILE "         $instrument BKG AREA SCALE FACTOR : ",$gv->PG_areafactor," (Only area taken into account)\n";
	if ($gv->PG_outerrad == 6000){print MYOUTFILE "         WARNING : $instrument regions : Maximum allowed radius reached";}
    }

#.. Produce background event list with sources removed to work out background rates: run edetect_chain, region and eveselect

    &get_bkg_filterset($event_list, $instrument);

#.. Produce Source and Background light Curves

    &PG_produce_lightcurve_file($gv->PG_src_region,$instrument,$event_list,$lc_src_outfile);  # Source LC    
    if ( ($gv->pn_interactive) || ($gv->mos1_interactive) || ($gv->mos2_interactive)) {
	    &PG_produce_lightcurve_file($gv->PG_bkg_region,$instrument,$event_list,$lc_bkg_outfile);} # BKG LC
	else {
	    &PG_produce_lightcurve_file("annulus",$instrument,$bkg_event_list,$lc_bkg_outfile);} # BKG LC
	
	
	

#.. Extraction of the Counts columns....

    &PG_extract_counts($lc_src_outfile,$lc_src_counts_file); # Source
    &PG_extract_counts($lc_bkg_outfile,$lc_bkg_counts_file); # BKG

#.. Run procedure to optimize S/N

    my $rate    = 1000.; # set to silly default value
    my $src_rad = 1000.; # set to silly default value
    my $max_sn  = 1000.; # set to silly default value 
#..    ($src_rad,$max_sn,$rate)=&PG_IDL_Optimize_SN($source_name,$instrument,$mode,$lc_src_counts_file,$lc_bkg_counts_file);
    ($src_rad,$max_sn,$rate)=&PG_Optimize_SN($source_name,$instrument,$mode,$lc_src_counts_file,$lc_bkg_counts_file);
#.. Set the values of the correspondent GTI_ parameters 
    if ($instrument eq "pn"){$gv->GTI_PN_Rate($rate);}      # Set the PN rate 
    if ($instrument eq "mos1"){$gv->GTI_MOS1_Rate($rate);}  # Set the MOS1 rate       
    if ($instrument eq "mos2"){$gv->GTI_MOS2_Rate($rate);}  # Set the MOS2 rate      

    &error_code(0);
    return(0);
}

#========================================================================
sub get_bkg_filterset(){

    my $event_list = $_[0];
    my $instr      = $_[1];

    chdir($gti_directory); 
    &run_edetectchain;
    chdir($gti_directory); 

    if ($instr eq "pn") {$expr="\#XMMEA_EP && (FLAG==0) && (PATTERN<=4) && region($bkg_regionset:REGION)";}
    if ($instr eq "mos1" or $instr eq "mos2") {$expr="\#XMMEA_EM && (FLAG==0) && (PATTERN<=12) && region($bkg_regionset:REGION)";}

   @args = ("region eventset=$event_list tempset=temp.ds srclisttab=$src_list operationstyle=global shrinkconfused=yes radiusstyle=contour bkgratestyle=col regionset=$src_regionset bkgregionset=$bkg_regionset");
    system(@args)== 0 or &error_code(10);

    @args = ("evselect table=$event_list filteredset=$bkg_event_list expression=\"$expr\"");
    system(@args)== 0 or &error_code(10);
    return(1);
}

#========================================================================
sub PG_produce_lightcurve_file(){
    
# Produce an event file for a given region and energy range
    
    my $E1         = $gv->PG_minpi;
    my $E2         = $gv->PG_maxpi;
    my $XC         = $gv->PG_sx;
    my $YC         = $gv->PG_sy;
    my $RSRC       = $gv->PG_source_rad;
    my $RBKG1      = $gv->PG_innerrad;
    my $RBKG2      = $gv->PG_outerrad;
    my $T          = $gv->PG_timebinning;
    my $tstart     = $gv->PG_timestart;
    my $tstop      = $gv->PG_timeend;
    my $region     = $_[0];
    my $instr      = $_[1];
    my $event_file = $_[2];
    my $outfile    = $_[3];

    if ($instr eq "pn") {$expr="\#XMMEA_EP && (FLAG==0) && (PATTERN<=4) && (PI>=$E1) && (PI<=$E2)";}
    if ($instr eq "mos1" or $instr eq "mos2") {$expr="\#XMMEA_EM && (FLAG==0) && (PATTERN<=12) && (PI>=$E1) && (PI<=$E2)";}
	
    if (lc($region) eq "circle") {$areaexpr="((X,Y) in circle($XC,$YC,$RSRC))";}
    if (lc($region) eq "annulus") {$areaexpr="((X,Y) in annulus($XC,$YC,$RBKG1,$RBKG2))";}
    print "\n   \#> Producting Light Curve Files ... \n";
    print "     $instr : $region ($areaexpr): Energy Range: $E1 - $E2 keV \n";
    print "     $instr : $region ($areaexpr): Time Bining : $T sec\n";
    print "     (Event File : $event_file )\n";
    print "     (Output File: $outfile) \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Producting Light Curve Files ... \n";}
    if ($Log_Flag == 0) {print MYOUTFILE "     $instr : $region ($areaexpr): Energy Range: $E1 - $E2 keV \n";}
    if ($Log_Flag == 0) {print MYOUTFILE "     $instr : $region ($areaexpr): Time Bining : $T sec\n";}
    if ($Log_Flag == 0) {print MYOUTFILE "     (Event File : $event_file )\n";}
    if ($Log_Flag == 0) {print MYOUTFILE "     (Output File: $outfile) \n";}
 
    @args = ("evselect table=$event_file:EVENTS withrateset=yes rateset=$outfile maketimecolumn=yes timecolumn=TIME timebinsize=$T expression=\"$expr \&\& $areaexpr\" makeratecolumn=no withtimeranges=yes timemin=$tstart timemax=$tstop");

    system(@args)== 0 or &error_code(10);
    
    return(1);
}

#========================================================================
sub PG_extract_counts(){
    
    my $event_file  = $_[0];
    my $out_file    = $_[1];

    @args = ("rm -f $out_file");
    system(@args)== 0 or &error_code(10);

    @args = ("fdump $event_file+1 $out_file \"COUNTS TIME\" - prhead=no showcol=no prdata=yes showunit=no showrow=no");
    system(@args)== 0 or &error_code(10);

    my $temp_out_file = "temp.txt";
    @args = ("grep -v \^\$ $out_file | awk '{print \$1\" \"\$2}' > $temp_out_file");
    system(@args);
    @args = ("mv -f $temp_out_file $out_file");
    system(@args);

    return(1);
}

#========================================================================
sub PG_Optimize_SN(){
   print "    PG_Optimize_SN:\n ";   
    my $source_name     = $_[0];
    my $instrument      = $_[1];
    my $frame           = $_[2];
    my $src_counts_file = $_[3];
    my $bkg_counts_file = $_[4];

    $T             = $gv->PG_timebinning;
    $RSRC          = $gv->PG_source_rad;
    $annulus[0]    = $gv->PG_innerrad;
    $annulus[1]    = $gv->PG_outerrad;
    $rectangle[0]  = 0.;
    $rectangle[1]  = 0.;

    my $bkg_lc_points;
    chomp($bkg_lc_points    = `grep -v \^\$ $bkg_counts_file | wc | awk '{print \$1}'`);
    my $src_lc_points;
    chomp($src_lc_points    = `grep -v \^\$ $src_counts_file | wc | awk '{print \$1}'`);

#.. Get vectors with source and background counts
    open(INPUT, "$src_counts_file") or die "Cant open output file: $! \n";    
    while (defined(my $line=<INPUT>)) {
	chomp($line);
	($cts,$t) = split/\s/,$line;
	push(@src_counts,$cts);
	push(@src_time,$t);
    }
    close(INPUT);
    open(INPUT, "$bkg_counts_file") or die "Cant open output file: $! \n";
    while(defined(my $line=<INPUT>)) {
	chomp($line);
	($cts,$t) = split/\s/,$line;
	push(@bkg_counts,$cts);
	push(@bkg_time,$t);
    }
    close(INPUT);
    
#.. Get array with bin time (should be the same length for src_counts and bkg_counts)
    for ($count = 0; $count <= $#src_counts; $count++){$time_bin_array[$count]=$T;}
    
#.. Get ratio of souce to background area
    my $area_ratio = $gv->PG_areafactor;

#.. Correct Background counts by BACKSCALE
    for ($count = 0; $count <= $#bkg_counts; $count++){$bkg_scaled_counts[$count]=$area_ratio*$bkg_counts[$count];}

#.. Workout Background corrected Source Count
    for ($count = 0; $count <= $#src_counts; $count++){$bkgclean_src_counts[$count]=$src_counts[$count]-$bkg_scaled_counts[$count];} 
    
#.. Workout Count Rates
    for ($count = 0; $count <= $#time_bin_array; $count++){
	$src_ctr[$count]=$src_counts[$count]/$time_bin_array[$count];
	$bkg_ctr[$count]=$bkg_counts[$count]/$time_bin_array[$count];
	$bkg_scaled_ctr[$count]=$bkg_scaled_counts[$count]/$time_bin_array[$count];
        $bkgclean_src_ctr[$count]=$bkgclean_src_counts[$count]/$time_bin_array[$count];	
    }

#.. Workout Cumulative Time Array
    $cont     = 1;
    while ($cont <= $#time_bin_array+1){
	$cont_in    = $cont;
	$time_cum   = 0;
	while ($cont_in <= $#time_bin_array+1){
	    $time_cum  = $time_cum + $time_bin_array[$cont_in-1];
	    $cont_in++;
	}
	$cum_time_bin_array[$cont-1]  = $time_cum - $T/2.0; # Place in center of Bin
	$cont++;
    }
    @cum_time_bin_array = reverse(@cum_time_bin_array);     # Sort out in increasing order
    
#.. Output to a file these values

    my $counts_file = "PG_counts.dat";
    open(COUNTS, "> $counts_file") or die "Cant open output file: $! \n";    #open for write, overwrite
     for ($count = 0; $count <= $#cum_time_bin_array; $count++){
	 $t_bin=$count+1;
	 printf COUNTS "%d %.1f %d %d %.6f %.6f \n",$t_bin,$cum_time_bin_array[$count],$src_counts[$count],$bkg_counts[$count],$bkg_scaled_counts[$count],$bkgclean_src_counts[$count];
     }
    close(COUNTS);
    $counts_desc_file = "PG_counts_desc_file.dat";
    open(SN_DESC, "> $counts_desc_file") or die "Cant open output file: $! \n";      #open for write, overwrite
    printf SN_DESC "CHANNEL B\n";
    printf SN_DESC "TIME E Time\n";
    printf SN_DESC "SRC E Counts\n";
    printf SN_DESC "BKG E Counts\n";
    printf SN_DESC "BKG_SC E counts\n";
    printf SN_DESC "BKG_CL E Counts\n";
    close(SN_DESC);
	$out_counts_file="PG_".$source_name."_".$instrument.".fits";
	
   &create_fits_table($counts_desc_file,$counts_file,$out_counts_file,"COUNTS");
    
    
    my $rates_file = "PG_rates.dat";
    open(RATES, "> $rates_file") or die "Cant open output file: $! \n";    #open for write, overwrite
     for ($count = 0; $count <= $#cum_time_bin_array; $count++){
	 $t_bin=$count+1;
	 printf RATES "%d %.1f %.6f %.6f %.6f %.6f \n",$t_bin,$cum_time_bin_array[$count],$src_ctr[$count],$bkg_ctr[$count],$bkg_scaled_ctr[$count],$bkgclean_src_ctr[$count];
     }
    close(RATES);
  	$rates_desc_file = "PG_rates_desc_file.dat";
    open(SN_DESC, "> $rates_desc_file") or die "Cant open output file: $! \n";      #open for write, overwrite
    printf SN_DESC "CHANNEL B\n";
    printf SN_DESC "TIME E Time\n";
    printf SN_DESC "SRC E Rate\n";
    printf SN_DESC "BKG E Rate\n";
    printf SN_DESC "BKG_SC E Rate\n";
    printf SN_DESC "BKG_CL E Rate\n";
    close(SN_DESC);
	$out_rates_file="PG_rates.fits";
	
   &create_fits_table($rates_desc_file,$rates_file,$out_rates_file,"RATES");   
   
#.. Sort out in increasing order counts and count rates, according to background count rate

    my $jcont=1; 
    while ($jcont <= $#bkg_counts+1){
	$lowest = $bkg_counts[$jcont-1];
	$icont=$jcont;
	while ($icont <= $#bkg_counts+1){
	    if ($bkg_counts[$icont-1] < $lowest){
		$lowest = $bkg_counts[$icont-1];
		
		$temp   = $src_counts[$jcont-1];
		$src_counts[$jcont-1] = $src_counts[$icont-1];
		$src_counts[$icont-1] = $temp;

		$temp   = $bkg_counts[$jcont-1];
		$bkg_counts[$jcont-1] = $bkg_counts[$icont-1];
		$bkg_counts[$icont-1] = $temp;

		$temp   = $bkg_scaled_counts[$jcont-1];
		$bkg_scaled_counts[$jcont-1] = $bkg_scaled_counts[$icont-1];
		$bkg_scaled_counts[$icont-1] = $temp;	    
		
		$temp   = $bkgclean_src_counts[$jcont-1];
		$bkgclean_src_counts[$jcont-1] = $bkgclean_src_counts[$icont-1];
		$bkgclean_src_counts[$icont-1] = $temp;
		
		$temp   = $src_ctr[$jcont-1];
		$src_ctr[$jcont-1] = $src_ctr[$icont-1];
		$src_ctr[$icont-1] = $temp;
		
		$temp   = $bkg_ctr[$jcont-1];
		$bkg_ctr[$jcont-1] = $bkg_ctr[$icont-1];
		$bkg_ctr[$icont-1] = $temp;	    
		
		$temp   = $bkg_scaled_ctr[$jcont-1];
		$bkg_scaled_ctr[$jcont-1] = $bkg_scaled_ctr[$icont-1];
		$bkg_scaled_ctr[$icont-1] = $temp;	    
		
		$temp   = $bkgclean_src_ctr[$jcont-1];
		$bkgclean_src_ctr[$jcont-1] = $bkgclean_src_ctr[$icont-1];
		$bkgclean_src_ctr[$icont-1] = $temp;	   
	    }
	    $icont++;
	} 
	$jcont++;
    }

#.. Workout cumulative count and count rate vectors and S/N vector (the final vectors have to be inverted)

    $cont     = 1;
    while ($cont <= $#src_counts+1){
	$cont_in          = $cont;
	$src_cum          = 0;
	$bkg_cum          = 0;
	$bkg_scaled_cum   = 0;
	$bkgclean_src_cum = 0;
	while ($cont_in <= $#src_counts+1){
	    $src_cum          = $src_cum + $src_counts[$cont_in-1];
	    $bkg_cum          = $bkg_cum + $bkg_counts[$cont_in-1];
	    $bkg_scaled_cum   = $bkg_scaled_cum + $bkg_scaled_counts[$cont_in-1];
	    $bkgclean_src_cum = $bkgclean_src_cum + $bkgclean_src_counts[$cont_in-1];
	    $cont_in++;
	}
	$cum_src_counts[$cont-1]          = $src_cum;
	$cum_bkg_counts[$cont-1]          = $bkg_cum;
	$cum_bkg_scaled_counts[$cont-1]   = $bkg_scaled_cum;
	$cum_bkgclean_src_counts[$cont-1] = $bkgclean_src_cum;
	$sn[$cont-1] = ($cum_src_counts[$cont-1]-$area_ratio*$cum_bkg_counts[$cont-1])
	    /sqrt($cum_src_counts[$cont-1]+$area_ratio*$area_ratio*$cum_bkg_counts[$cont-1]); 
	$cont++;
    }

    @cum_src_counts          = reverse(@cum_src_counts);
    @cum_bkg_counts          = reverse(@cum_bkg_counts);
    @cum_bkg_scaled_counts   = reverse(@cum_bkg_scaled_counts);
    @cum_bkgclean_src_counts = reverse(@cum_bkgclean_src_counts);
    @sn                      = reverse(@sn);

    for ($count = 0; $count <= $#cum_src_counts; $count++){
	$cum_src_ctr[$count]=$cum_src_counts[$count]/$cum_time_bin_array[$count];
	$cum_bkg_ctr[$count]=$cum_bkg_counts[$count]/$cum_time_bin_array[$count];  
	$cum_bkg_scaled_ctr[$count]=$cum_bkg_scaled_counts[$count]/$cum_time_bin_array[$count];
        $cum_bkgclean_src_ctr[$count]=$cum_bkgclean_src_counts[$count]/$cum_time_bin_array[$count];	
    }

 #.. Percentage vectors

    $max_sn = 0;
    $cont   = 1;
    while ($cont <= $#sn+1){                   # get maximum s/n
	$i = $sn[$cont-1];
	if ($i > $max_sn){$max_sn = $i; $max_sn_index = $cont-1;}
	$cont++;
    }
    $max_t = 0;
    $cont   = 1;
    while ($cont <= $#cum_time_bin_array+1){   # get maximum time
	$i = $cum_time_bin_array[$cont-1];
	if ($i > $max_t){$max_t = $i; $max_t_index = $cont-1;}
	$cont++;
    } 

    $cont   = 1;
    while ($cont <= $#sn+1){  
	$per_sn[$cont-1]                     = 100.0*($sn[$cont-1]/$max_sn);                       # Signal to Noise
	$per_cum_time_bin_array[$cont-1]     = 100.0*($cum_time_bin_array[$cont-1]/$max_t);
	$per_ratio_bkg_src_counts[$cont-1]   = 100.0*($cum_bkg_scaled_counts[$cont-1]/$cum_bkgclean_src_counts[$cont-1]);        # Ratio Background to Source
	print "$cum_bkg_scaled_counts[$cont-1]  $cum_bkgclean_src_counts[$cont-1] $per_ratio_bkg_src_counts[$cont-1]\n";
	$per_bkgclean_src_counts[$cont-1]    = 100.0*($cum_bkgclean_src_counts[$cont-1]/$cum_bkgclean_src_counts[$max_t_index]); # Source Counts
	$per_bkg_scaled_counts[$cont-1]      = 100.0*($cum_bkg_scaled_counts[$cont-1]/$cum_bkg_scaled_counts[$max_t_index]);     # Background Counts
	$per_bkgclean_src_counts_te[$cont-1] = $cum_time_bin_array[$cont-1]*($cum_bkgclean_src_counts[$max_t_index]/$max_t);     # Source Counts Theoretical
	$cont++;
    }

    $cont   = 1;
    while ($cont <= $#sn+1){ 
	$per_per_ratio_bkg_src_counts[$cont-1]   = 100.0*($per_ratio_bkg_src_counts[$cont-1]/$per_ratio_bkg_src_counts[$max_t_index]); # Background to Source
	print "$per_ratio_bkg_src_counts[$max_t_index] $per_per_ratio_bkg_src_counts[$cont-1]\n";
	$per_per_bkgclean_src_counts_te[$cont-1] = 100.0*($per_bkgclean_src_counts_te[$cont-1]/$per_bkgclean_src_counts_te[$max_t_index]);         
	$cont++;
    }

#.. Output Some Information

    print "   \#> PG Summary  \n";
    print "      Max S/N                                       : $max_sn \n";
    print "      Optimum Background Count Rate Cut (cts/sec)   : $bkg_ctr[$max_sn_index]\n";
    print "       At this Background Count Rate Cut: \n"; 
    print "        Source Radius (image units)                 : $RSRC \n";
    print "        Inner Annulus Radius (image units)          : $annulus[0]\n";
    print "        Outer Annulus Radius (image units)          : $annulus[1]\n";
    print "        Ratio Source/BKG Region                     : $area_ratio\n";
    print "        Counts Source Region (cts)                  : $cum_src_counts[$max_sn_index]\n";
    print "        Total Background Counts (cts)               : $cum_bkg_counts[$max_sn_index]\n";
    print "        Background Counts Source Region(cts)        : $cum_bkg_scaled_counts[$max_sn_index]\n";
    print "        Exposure Time (sec)                         : $cum_time_bin_array[$max_sn_index]\n";

    my $sn_file = "PG_sn.dat";
    open(SN, "> $sn_file") or die "Cant open output file: $! \n";      #open for write, overwrite
      for ($count = 0; $count <= $#time_bin_array; $count++){
         $t_bin=$count+1;
         printf SN "%d %.6f %.6f %.6f \n",$t_bin,$bkg_ctr[$count],$sn[$count],$per_ratio_bkg_src_counts[$count]
     }
    close(SN);
#.. Create FITS Table
#.. We use the standard ftool program "fcreate" to create a fits file from the ASCII file
#.. First we have to create a file with the column description file called 
    $sn_desc_file = "PG_sn_desc_file.dat";
    open(SN_DESC, "> $sn_desc_file") or die "Cant open output file: $! \n";      #open for write, overwrite
    printf SN_DESC "CHANNEL B\n";
    printf SN_DESC "BKG E Rate\n";
    printf SN_DESC "SN E Signal2Noise\n";
    printf SN_DESC "ratio E \n";
    close(SN_DESC);
	$out_sn_file="PG_sn.fits";
	
   &create_fits_table($sn_desc_file,$sn_file,$out_sn_file,"Signal2Noise");    
    
    my $per_file = "PG_percentages.dat";
    open(PER, "> $per_file") or die "Cant open output file: $! \n";    #open for write, overwrite
     for ($count = 0; $count <= $#time_bin_array; $count++){
	 $t_bin=$count+1;
	 printf PER "%.6f %.6f %.6f %.6f %.6f\n",$bkg_ctr[$count],$per_per_ratio_bkg_src_counts[$count],$per_bkgclean_src_counts[$count],$per_bkg_scaled_counts[$count],$per_per_bkgclean_src_counts_te[$count]
     }
    close(PER);

 	$percent_desc_file = "PG_percentages_desc_file.dat";
    open(SN_DESC, "> $percent_desc_file") or die "Cant open output file: $! \n";      #open for write, overwrite   
    printf SN_DESC "BKG E Rate\n";
    printf SN_DESC "PER E Percentage\n";
    printf SN_DESC "PER_BKGCL E Percentage\n";    
    printf SN_DESC "PER_BKGSC E Percentage\n";
    printf SN_DESC "PP_BKGCL E Percentage\n";
    close(SN_DESC);
	$out_per_file="PG_percentages.fits";
	
   &create_fits_table($percent_desc_file,$per_file,$out_per_file,"PERCENT");

   &merge_fits_files($out_counts_file,$out_rates_file,$out_sn_file,$out_per_file);

    return($RSRC,$max_sn,$bkg_ctr[$max_sn_index]);
}
#========================================================================
sub create_fits_table{
# Use files: PG_rates(1st,2nd plots) dat PG_sn.dat(3rd,4th plots) PG_percentages.dat(5rd,6th,7th,8th plots
#..    fcreate TMPJNKtempl TMPJNKdata TMPJNKfits2 extname="PG"; 
    print "Creating FITS files\n ";
    $desc_file = $_[0];
    $sn_file = $_[1];
    $out_file= $_[2];   
    $ext_name = $_[3];
    @args = ("fcreate $desc_file $sn_file $out_file clobber=yes extname=$ext_name" ) ;
    system(@args) == 0 or &error_code(10);    
    
    return(1);
}

#========================================================================
#========================================================================
sub merge_fits_files{
# Use files: PG_rates(1st,2nd plots) dat PG_sn.dat(3rd,4th plots) PG_percentages.dat(5rd,6th,7th,8th plots
#..    fcreate TMPJNKtempl TMPJNKdata TMPJNKfits2 extname="PG"; 
      print "Merging FITS files\n ";
    $file1 = $_[0];
    $file2 = $_[1]."[1]";
    $file3= $_[2]."[1]";   
    $file4 = $_[3]."[1]";

	$file_append=$file1."[1]";
	print "KKKKK $file_append\n";
	print "KKKKK $file2\n";
    @args = ("fappend $file2 $file_append" ) ;
    system(@args) == 0 or &error_code(10);
    $file_append=$file1."[2]";
    print "KKKKK $file_append\n";
    print "KKKKK $file3\n";
    @args = ("fappend $file3 $file_append" ) ;
    system(@args) == 0 or &error_code(10);
    $file_append=$file1."[3]";
    print "KKKKK $file_append\n";
    print "KKKKK $file4\n";
    @args = ("fappend $file4 $file_append" ) ;
    system(@args) == 0 or &error_code(10);    
 
    return(1);
}

#========================================================================
sub PG_IDL_Optimize_SN(){
    
    my $source_name     = $_[0];
    my $instrument      = $_[1];
    my $frame           = $_[2];
    my $src_counts_file = $_[3];
    my $bkg_counts_file = $_[4];

    my $lc_points;
    chomp($lc_points    = `grep -v \^\$ $bkg_counts_file | wc | awk '{print \$1}'`);

    my $opt = 0;
    if ($frame eq "PrimeFullWindowExtended" || $frame eq "PrimeFullWindow") {
	$opt = 1;
    } elsif ($frame eq "PrimeLargeWindow") {
	$opt = 2;
    } elsif ($frame eq "PrimeSmallWindow" || $frame eq "PrimePartialW2") {
	$opt = 3;
    }
 
    $T             = $gv->PG_timebinning;
    $RSRC          = $gv->PG_source_rad;
    $annulus[0]    = $gv->PG_innerrad;
    $annulus[1]    = $gv->PG_outerrad;
    $rectangle[0]  = 0.;
    $rectangle[1]  = 0.;
    
    my $idl_init_file     = $Script_directory."xmm_idl_init.pro";
    my $idl_logfile_name  = $gti_directory."PG_gti_new.log";

    my $templatefile_name  = $Script_directory."IDL\_gti\_new\.template";      # Master template file
    my $scriptfile_name    = $Script_directory."default\.pro";
    if ($instrument eq "pn")   {$scriptfile_name = $pnidl_scriptfile_name;}           
    if ($instrument eq "mos1") {$scriptfile_name = $m1idl_scriptfile_name;}           
    if ($instrument eq "mos2") {$scriptfile_name = $m2idl_scriptfile_name;} 

    open MASTER_TEMPLATE,  "<$templatefile_name";
    open OUTPUT,           ">$scriptfile_name ";
    
    while (defined(my $template_line=<MASTER_TEMPLATE>)) {
	chomp($template_line);
	$template_line =~ s/%P1\b/$source_name/g;
	$template_line =~ s/%P2\b/PG/g;
    	$template_line =~ s/%P3\b/$lc_points/g;
    	$template_line =~ s/%P4\b/$src_counts_file/g;
    	$template_line =~ s/%P5\b/$bkg_counts_file/g;
    	$template_line =~ s/%P6\b/$T/g;
    	$template_line =~ s/%P7\b/$opt/g;
    	$template_line =~ s/%P8\b/$RSRC/g;
    	$template_line =~ s/%P9\b/0/g;
    	$template_line =~ s/%P10\b/$rectangle[0]/g;
    	$template_line =~ s/%P11\b/$rectangle[1]/g;
    	$template_line =~ s/%P12\b/$annulus[0]/g;
    	$template_line =~ s/%P13\b/$annulus[1]/g;    	
	$template_line =~ s/%P14\b/$instrument/g;
	$template_line =~ s/%P15\b/$idl_logfile_name/g;
	print OUTPUT "$template_line \n";
    }
    close(OUTPUT);
    close(MASTER_TEMPLATE);

    @args = ("idl $idl_init_file $scriptfile_name");
    system(@args) == 0 or &error_code(10);

#. Read IDL log file
    $src_rad       = $RSRC; # default
    $max_sn        = 1000.; # default
    $bkg_count_rate= 1000.; # default

    open INFO,  "<$idl_logfile_name";
    while (defined(my $template_line=<INFO>)) {
	chomp($template_line);
	@split = split/:/,$template_line;
	if ($template_line =~ m/^SourceRadius/i){$src_rad=$split[1]; $src_rad =~ s/\s+//;}
	if ($template_line =~ m/^Max_SN/i){$max_sn=$split[1]; $max_sn =~ s/\s+//;}
	if ($template_line =~ m/^CountRate/i){$bkg_count_rate=$split[1]; $bkg_count_rate=~ s/\s+//;}
    }
    close(INFO);

    return($src_rad,$max_sn,$bkg_count_rate);
}
1;
