## @file
# Master file for producing event list for EPIC, RGS and OM

## @method void produce_eventlist()
# Starts and controls the production of the Event Lists for EPIC, RGS and OM.
# @return Void

#========================================================================
sub produce_eventlist(){
#.. Run epproc, emproc and rgsproc
    print "\n----> Start Analysis <----\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Start Analysis <----\n";}

	if (($gv->PN eq "yes") || ($gv->MOS eq "yes")) { 			
		@epic_files = &produce_epiceventlist; 
	} else {$epic_flag_msg = "NOT DONE"; }
	
	if ($gv->RGS eq "yes"){  
		@rgs_files  = &produce_rgseventlist; 		
	} else { $rgs_flag_msg  = "NOT DONE"; }

	if($gv->OM eq "yes"){ @om_files   = &produce_omeventlist; }


#.. Check if event files exist. As long as one exist, keep going. Set instr_flag to TRUE.    
    $epic_flag     = 0;
    $epic_flag_msg = "FAILED";
    foreach (@epic_files){if ($_){$epic_flag = 1; $epic_flag_msg = "OK";}}
    $rgs_flag      = 0;
    $rgs_flag_msg  = "FAILED";
    foreach (@rgs_files){if ($_){$rgs_flag = 1; $rgs_flag_msg  = "OK";}}
    $om_flag       = 0;
    $om_flag_msg   = "FAILED";
    foreach (@om_files){if ($_){$om_flag = 1; $om_flag_msg  = "OK";}}

    print "\n----> Summary of Analysis <---- \n";
    print "   \#> EPIC Analysis Flag : $epic_flag_msg \n"; 
    print "   \#> RGS Analysis Flag  : $rgs_flag_msg \n";
    print "   \#> OM Analysis Flag   : $om_flag_msg \n";
    print "------------------------------- \n";
    if ($Log_Flag == 0) {
	    print MYOUTFILE "\n----> Summary of Analysis <---- \n";
	    print MYOUTFILE "   \#> EPIC Analysis Flag : $epic_flag_msg \n"; 
	    print MYOUTFILE "   \#> RGS Analysis Flag  : $rgs_flag_msg \n";
	    print MYOUTFILE "   \#> OM Analysis Flag   : $om_flag_msg \n";
	    print MYOUTFILE "------------------------------- \n";
    }

    if (!$epic_flag && !$rgs_flag && !$om_flag){    # If no event files exist for EPIC, RGS or OM, return error code and exit
	&error_code(3);
    } elsif (!$epic_flag) {                         # If no event files exist for EPIC, return error code and exit
	&error_code(3);
    } else {
	&error_code(0);
    }

    return(0);
}
1;
