## @file
# Produce EPIC point source lightcurve

## @method void produce_lightcurve()
# Produce EPIC lightcurve products
# @return Void

#========================================================================
sub produce_lightcurve(){
    print "\n----> Working out Light Curve .....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Working out Light Curve ....\n";}
    

#.. Define Global Variables
    print "   \#> Defining File Names for Light Curve ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for Light Curve ..... ";}

    use vars qw($src_scale 
		$bkg_scale
		$pn_obsmode
		$m1_obsmode
		$m2_obsmode
		$pn_event_list
		$m1_event_list
		$m2_event_list
		$pn_GTI_file
		$m1_GTI_file
		$m2_GTI_file
		$pn_bkglc_file 
		$m1_bkglc_file 
		$m2_bkglc_file 
		$pn_gti_cleanevents_file
		$pn_gti_image_file      
		$pn_gti_smoothimage_file
		$pn_eregionanal_file
		$pn_lcurve_region_file
		$pn_lcurve_bkgregion_file
		$pn_lcurve_combinedregion_file
		$pn_emllist_file
		$source_pnregion_file
		$m1_gti_cleanevents_file         
		$m1_gti_image_file               
		$m1_gti_smoothimage_file 
		$m1_eregionanal_file
		$m1_lcurve_region_file
		$m1_lcurve_bkgregion_file
		$m1_lcurve_combinedregion_file
		$m1_emllist_file
		$source_m1region_file
		$m2_gti_cleanevents_file         
		$m2_gti_image_file               
		$m2_gti_smoothimage_file   
		$m2_eregionanal_file
		$m2_lcurve_region_file
		$m2_lcurve_bkgregion_file
		$m2_lcurve_combinedregion_file
		$m2_emllist_file
		$source_m2region_file
		$pn_sourcelc_e0_file         		
		$pn_bkglc_e0_file            
		$pn_srcbkgsubtractlc_e0_file 
		$m1_sourcelc_e0_file        		
		$m1_bkglc_e0_file            
		$m1_srcbkgsubtractlc_e0_file 
		$m2_sourcelc_e0_file         		
		$m2_bkglc_e0_file            
		$m2_srcbkgsubtractlc_e0_file   
		$pn_lcurve_flag
		$m1_lcurve_flag
		$m2_lcurve_flag
		);

#.. Check if the user has defined its own coordinates
	&setcoordinates();	
	  
    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $source_ra    = $gv->SRC_RA;
    my $source_dec   = $gv->SRC_DEC;

#.. Definition of file names
    $pn_gti_cleanevents_file         = $gti_directory."pn_events_gtifiltered\.fit";  		       
    $pn_gti_image_file               = $gti_directory."pn_img_gtifiltered\.img";			       
    $pn_gti_smoothimage_file         = $gti_directory."pn_img_gtifiltered_smooth\.img";
    $pn_eregionanal_file             = $lcurve_directory.$source_name."_pn_eregionanalyse\.txt";       
    $pn_emllist_file                 = $images_directory."pn_emllist\.fits";
    $source_pnregion_file            = $images_directory.$source_name."_pnregion\.reg";
    $pn_lcurve_region_file           = $lcurve_directory.$source_name."_pn_region\.reg";
    $pn_lcurve_bkgregion_file        = $lcurve_directory.$source_name."_pn_bkgregion\.reg";
    $pn_lcurve_combinedregion_file   = $lcurve_directory.$source_name."_pn_combinedregion\.reg";
    $m1_gti_cleanevents_file         = $gti_directory."m1_events_gtifiltered\.fit";  		       	  
    $m1_gti_image_file               = $gti_directory."m1_img_gtifiltered\.img";			       	
    $m1_gti_smoothimage_file         = $gti_directory."m1_img_gtifiltered_smooth\.img";
    $m1_eregionanal_file             = $lcurve_directory.$source_name."_m1_eregionanalyse\.txt";       
    $m1_emllist_file                 = $images_directory."m1_emllist\.fits";
    $source_m1region_file            = $images_directory.$source_name."_m1region\.reg";
    $m1_lcurve_region_file           = $lcurve_directory.$source_name."_m1_region\.reg";
    $m1_lcurve_bkgregion_file        = $lcurve_directory.$source_name."_m1_bkgregion\.reg";
    $m1_lcurve_combinedregion_file   = $lcurve_directory.$source_name."_m1_combinedregion\.reg";
    $m2_gti_cleanevents_file         = $gti_directory."m2_events_gtifiltered\.fit";  		       	  
    $m2_gti_image_file               = $gti_directory."m2_img_gtifiltered\.img";			       	
    $m2_gti_smoothimage_file         = $gti_directory."m2_img_gtifiltered_smooth\.img";
    $m2_eregionanal_file             = $lcurve_directory.$source_name."_m2_eregionanalyse\.txt";       
    $m2_emllist_file                 = $images_directory."m2_emllist\.fits";
    $source_m2region_file            = $images_directory.$source_name."_m2region\.reg";
    $m2_lcurve_region_file           = $lcurve_directory.$source_name."_m2_region\.reg";
    $m2_lcurve_bkgregion_file        = $lcurve_directory.$source_name."_m2_bkgregion\.reg";
    $m2_lcurve_combinedregion_file   = $lcurve_directory.$source_name."_m2_combinedregion\.reg";
 
    $pn_sourcelc_e0_file             = $lcurve_directory.$source_name."_pnsource.lc";
    $pn_bkglc_e0_file                = $lcurve_directory.$source_name."_pnbkg.lc";
    $pn_srcbkgsubtractlc_e0_file     = $lcurve_directory.$source_name."_pnsourcebkgsubtracted.lc";
    $m1_sourcelc_e0_file             = $lcurve_directory.$source_name."_m1source.lc";
    $m1_bkglc_e0_file                = $lcurve_directory.$source_name."_m1bkg.lc";
    $m1_srcbkgsubtractlc_e0_file     = $lcurve_directory.$source_name."_m1sourcebkgsubtracted.lc";
    $m2_sourcelc_e0_file             = $lcurve_directory.$source_name."_m2source.lc";
    $m2_bkglc_e0_file                = $lcurve_directory.$source_name."_m2bkg.lc";
    $m2_srcbkgsubtractlc_e0_file     = $lcurve_directory.$source_name."_m2sourcebkgsubtracted.lc";

    $pn_bkglc_file                   = $lcurve_directory.$source_name."_pnbkg.lc";
    $m1_bkglc_file                   = $lcurve_directory.$source_name."_m1bkg.lc";
    $m2_bkglc_file                   = $lcurve_directory.$source_name."_m2bkg.lc";

# Light curve parameters

    $pn_lcurve_flag    = 1; # Flag that determines whether the light curve should be worked out (1: True)
    $m1_lcurve_flag    = 1; # Flag that determines whether the light curve should be worked out (1: True)
    $m2_lcurve_flag    = 1; # Flag that determines whether the light curve should be worked out (1: True)

# Check if the user select MOS1, MOS2, PN for analysing data
	if ($gv->PN eq "yes") { $pn_lcurve_flag = 1;}
	if ($gv->MOS eq "yes") { $m1_lcurve_flag = 1;}
	if ($gv->MOS eq "yes") { $m2_lcurve_flag = 1;}

    print " OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE " OK \n";}

    if ($Log_Spec_Flag == 0) {$temp_Log_Spec_Flag=$Log_Spec_Flag; $Log_Spec_Flag = 1;} # Deactivate Spectral Log flag if active

    chdir ($lcurve_directory);
    
#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {print "      Run cifbuild first \n"; &error_code(7);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {	
	&run_odfingest();
    }
    @test_odfingest=&test_odfingest();
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(7);}
    &set_odfingest($odfingestfile);

   
#.. Print sas_setup    
    &print_sas_setup();

#.. RA & DEC Coordinates of position to analyze

#.. If run_edetect chain has been ran, use the closest source to the RA DEC given by the user for analysis
    my ($ra_pnclosesource, $dec_pnclosesource, $pnmindist_deg) = &find_closest_source($source_pnregion_file);
    my ($ra_m1closesource, $dec_m1closesource, $m1mindist_deg) = &find_closest_source($source_m1region_file);
    my ($ra_m2closesource, $dec_m2closesource, $m2mindist_deg) = &find_closest_source($source_m2region_file);
    if ($pnmindist_deg != 1000. && $pnmindist_deg <= $gv->EDC_sourcematch){
	print "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to pn).\n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";}
	$ra_anal  = $ra_pnclosesource;
	$dec_anal = $dec_pnclosesource;
    } elsif ($m1mindist_deg != 1000. && $m1mindist_deg <= $gv->EDC_sourcematch) {
	print "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to mos1).\n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";}
	$ra_anal  = $ra_m1closesource;
	$dec_anal = $dec_m1closesource;
    } elsif ($m2mindist_deg != 1000. && $m2mindist_deg <= $gv->EDC_sourcematch) { 
	print "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to mos2).\n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";}
	$ra_anal  = $ra_m2closesource;
	$dec_anal = $dec_m2closesource;
    } else {
	$ra_anal  = $source_ra;     # (deg) User Coordinates
	$dec_anal = $source_dec;    # (deg) User coordinates
    }
    print "   \#> RA & DEC of source to analyse: \n";
    print "        RA : $ra_anal deg \n";
    print "        DEC: $dec_anal deg \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> RA & DEC of source to analyse: \n";
	print MYOUTFILE "        RA : $ra_anal deg \n";
	print MYOUTFILE "        DEC: $dec_anal deg \n";
    }

#.. Test for existance of GTI files
    if ($Log_GTIEvt_Flag == 0) {
	my $gtievent_log=$results_directory."gtievent_".$source_name."\.log";
	if (-e $gtievent_log) {unlink("$gtievent_log") or die "Cannot rm file: $!";}    
	open(GTILOG, "> $gtievent_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

    @pnGTI=&test_pngtifile();
    if ($pnGTI[0]){
	@pninfoGTI=&gti_file_inf($pn_GTI_file,"pn");
	if ($pninfoGTI[0] != 0.0){$pn_GTI_file=$pnGTI[1];} else {$pn_lcurve_flag = 0;}
    }  else {$pn_lcurve_flag = 0;}  
 
    @m1GTI=&test_mos1gtifile();
    if ($m1GTI[0]){
	@m1infoGTI=&gti_file_inf($m1_GTI_file,"mos1");
        if ($m1infoGTI[0] != 0.0) {$m1_GTI_file=$m1GTI[1];} else {$m1_lcurve_flag = 0;} 
    } else {$m1_lcurve_flag = 0;} 

    @m2GTI=&test_mos2gtifile();
    if ($m2GTI[0]){
	@m2infoGTI=&gti_file_inf($m2_GTI_file,"mos2");
	if ($m2infoGTI[0] != 0.0) {$m2_GTI_file=$m2GTI[1];} else {$m2_lcurve_flag = 0;} 
    } else {$m2_lcurve_flag = 0;}

    if ($Log_GTIEvt_Flag == 0) {
	close(GTILOG);
	$Log_GTIEvt_Flag = 1;	
    }

#.. Test for existance of PN and MOS event files (both Imaging and Timing)
    if ($Log_Evt_Flag == 0) {
	my $event_log=$results_directory."event_".$source_name."\.log";
	if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}    
	open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    } 

    @test_pnIevt=&test_pneventfiles();                                              # PN Imaging
    @test_pnTevt=&test_pntimingeventfiles();                                        # PN Timing
    if ($test_pnIevt[0]) {
	$pn_event_list=$test_pnIevt[1];
	&event_file_inf($pn_event_list,"pn");
    } elsif ($test_pnTevt[0]) { 
	$pn_event_list=$test_pnTevt[1];
	&event_file_inf($pn_event_list,"pn");
    } else {$pn_lcurve_flag = 0;}
    
    @test_m1Ievt=&test_mos1eventfiles();                                            # MOS1 Imaging
    @test_m1Tevt=&test_mos1timingeventfiles();                                      # MOS1 Timing
    if ($test_m1Tevt[0]) {
	$m1_event_list=$test_m1Tevt[1];
	&event_file_inf($m1_event_list,"mos1");
    } elsif ($test_m1Ievt[0]) {
	$m1_event_list=$test_m1Ievt[1];
	&event_file_inf($m1_event_list,"mos1");
    } else {$m1_lcurve_flag = 0;}
    
    @test_m2Ievt=&test_mos2eventfiles();                                            # MOS2 Imaging
    @test_m2Tevt=&test_mos2timingeventfiles();                                      # MOS2 Timing
    if ($test_m2Tevt[0]) {
	$m2_event_list=$test_m2Tevt[1];
	&event_file_inf($m2_event_list,"mos2");
    } elsif($test_m2Ievt[0]) {
	$m2_event_list=$test_m2Ievt[1];
	&event_file_inf($m2_event_list,"mos2");
    } else {$m2_lcurve_flag = 0;}

#.. Get instrument's observing mode and prepare things according to shape of source and background regions.  
    $pn_obsmode = $obsid_instruments{$source_obsID}{'pn'}{'mode'};
    $m1_obsmode = $obsid_instruments{$source_obsID}{'mos1'}{'mode'};
    $m2_obsmode = $obsid_instruments{$source_obsID}{'mos2'}{'mode'};
    $pn_obssubmode = $obsid_instruments{$source_obsID}{'pn'}{'submode'};
    $m1_obssubmode = $obsid_instruments{$source_obsID}{'mos1'}{'submode'};
    $m2_obssubmode = $obsid_instruments{$source_obsID}{'mos2'}{'submode'};
    
 
#.. Create GTI filtered event lists.
    if ($pn_lcurve_flag) 
    {&produce_gticleanfile($pn_event_list,$pn_gti_cleanevents_file,$pn_GTI_file,$pn_obsmode,"pn");}  # PN 
    if ($m1_lcurve_flag) 
    {&produce_gticleanfile($m1_event_list,$m1_gti_cleanevents_file,$m1_GTI_file,$m1_obsmode,"mos");} # MOS1
    if ($m2_lcurve_flag) 
    {&produce_gticleanfile($m2_event_list,$m2_gti_cleanevents_file,$m2_GTI_file,$m2_obsmode,"mos");} # MOS2

#.. Get some information out of the GTI filtered event lists.
    print "   \#> Information from GTI Filtered Event Files: \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Information from GTI Filtered Event Files: \n";}
    if ($pn_lcurve_flag) {&event_file_inf($pn_gti_cleanevents_file,"pn");}    # PN
    if ($m1_lcurve_flag) {&event_file_inf($m1_gti_cleanevents_file,"mos1");}  # MOS1
    if ($m2_lcurve_flag) {&event_file_inf($m2_gti_cleanevents_file,"mos2");}  # MOS2

    if ($Log_Evt_Flag == 0) {
	close(EVTLOG);
	$Log_Evt_Flag = 1;	
    }

#.. Make an image using the filtered event file to get source centroid from eregionanalyse
    if ($pn_lcurve_flag) # && !-e $pn_gti_smoothimage_file) 
    {&produce_smooth_image($pn_gti_cleanevents_file,$pn_gti_image_file,$pn_gti_smoothimage_file,"pn");}  # PN
    if ($m1_lcurve_flag) # && !-e $m1_gti_smoothimage_file) 
    {&produce_smooth_image($m1_gti_cleanevents_file,$m1_gti_image_file,$m1_gti_smoothimage_file,"mos1");} # MOS1
    if ($m2_lcurve_flag) # && !-e $m2_gti_smoothimage_file) 
    {&produce_smooth_image($m2_gti_cleanevents_file,$m2_gti_image_file,$m2_gti_smoothimage_file,"mos2");} # MOS2

#.. Using the instrument's image to work out the source centroid. Only for Imaging.

    print "   \#> Centroid of source to analyze (Physical Units) \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> Centroid of source to analyze (Physical Units) \n";
    }

    if ($pn_lcurve_flag) {

	if ($pn_obsmode eq 'IMAGING' || $pn_obsmode eq 'imaging') {
#.. Get shape of src and bkg regions:
	    $src_region_shape =  $gv->LC_pn_srcregion;
	    $bkg_region_shape =  $gv->LC_pn_bkgregion;
	    
#.. Run eregionanalyze at the source ra and dec position.
	    ($pn_bore_img_x,$pn_bore_img_y
	     ,$pn_bore_optrad_asec,$pn_bore_optradius 
	     ,$pn_bore_img_xbkg,$pn_bore_img_ybkg
	     ,$pn_bkg_inner_rad_asec,$pn_bkg_inner_rad
	     ,$pn_bkg_outer_rad_asec,$pn_bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);
	    ($pn_bore_img_x,$pn_bore_img_y
	     ,$pn_bore_optrad_asec,$pn_bore_optradius
	     ,$pn_bore_img_xbkg,$pn_bore_img_ybkg
	     ,$pn_bkg_inner_rad_asec,$pn_bkg_inner_rad
	     ,$pn_bkg_outer_rad_asec,$pn_bkg_outer_rad)
		=&run_eregionanalyse($pn_gti_image_file,$ra_anal,$dec_anal,"pn",$pn_obssubmode,$pn_eregionanal_file,"LC"); # PN 


	    if ($pn_bore_optrad_asec == 300){$pn_lcurve_flag = 0;} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

#.. Check values of parameters defining source and background regions
	    $src_XC = $gv->LC_pn_srcregion_X;
	    $src_YC = $gv->LC_pn_srcregion_Y;
	    $src_R1 = $gv->LC_pn_srcregion_R1;
	    $src_R2 = $gv->LC_pn_srcregion_R2;
	    $src_XL = $gv->LC_pn_srcregion_XLength;
	    $src_YL = $gv->LC_pn_srcregion_YLength;
	    $bkg_XC = $gv->LC_pn_bkgregion_X;
	    $bkg_YC = $gv->LC_pn_bkgregion_Y;
	    $bkg_R1 = $gv->LC_pn_bkgregion_R1;
	    $bkg_R2 = $gv->LC_pn_bkgregion_R2;
	    $bkg_XL = $gv->LC_pn_bkgregion_XLength;
	    $bkg_YL = $gv->LC_pn_bkgregion_YLength;

	    if ($src_XC == 0 && $src_YC == 0) {  # if values are set to cero take the default; from eregionanalyse 
		$gv->LC_pn_srcregion_X($pn_bore_img_x);
		$gv->LC_pn_srcregion_Y($pn_bore_img_y);		    
	    }

	    if ($bkg_XC == 0 && $bkg_YC == 0) { # if values are set to cero take the default; from eregionanalyse 
		$gv->LC_pn_bkgregion_X($pn_bore_img_xbkg);
		$gv->LC_pn_bkgregion_Y($pn_bore_img_ybkg);	
	    }

#.. SRC Region
	    if ($src_region_shape eq "circle") {
		$gv->LC_pn_srcregion_R1(0); 
		if ($src_R2 == 0){              # if values are set to cero take the default; from eregionanalyse
		    $gv->LC_pn_srcregion_R2($pn_bore_optradius); 
		}
		$src_area = $pi*$gv->LC_pn_srcregion_R2*$gv->LC_pn_srcregion_R2
		    - $pi*$gv->LC_pn_srcregion_R1*$gv->LC_pn_srcregion_R1; 		
	    } elsif ($src_region_shape eq "annulus") {
		if ($src_R1 == 0 || $src_R2 == 0) {
		    print "       PN SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN SRC Region should be defined \n";}
		    &error_code(7);
		}
		$src_area = $pi*$gv->LC_pn_srcregion_R2*$gv->LC_pn_srcregion_R2
		    - $pi*$gv->LC_pn_srcregion_R1*$gv->LC_pn_srcregion_R1; 	
	    } elsif ($src_region_shape eq "box") {
		if ($src_XL == 0 || $src_YL == 0) {
		    print "       PN SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN SRC Region should be defined \n";}
		    &error_code(7);
		}		
		$src_area = $gv->LC_pn_srcregion_XLength*$gv->LC_pn_srcregion_YLength;
	    } else {
		print "       PN SRC Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       PN SRC Region not recognized \n";}
		&error_code(7);
	    }

#.. BKG Region	    
	    if ($bkg_region_shape eq "circle") { 		
		$gv->LC_pn_bkgregion_R1(0);
		if ($bkg_R2 == 0){ 
		    print "       PN BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN BKG Region should be defined \n";}
		    &error_code(7);
		}
		$bkg_area = $pi*$gv->LC_pn_bkgregion_R2*$gv->LC_pn_bkgregion_R2
		    - $pi*$gv->LC_pn_bkgregion_R1*$gv->LC_pn_bkgregion_R1; 
	    } elsif ($bkg_region_shape eq "annulus") {
		if ($bkg_R1 == 0 || $bkg_R2 == 0) {      # if values are set to cero take the default; from eregionanalyse
		    $gv->LC_pn_bkgregion_R1($pn_bkg_inner_rad);
		    $gv->LC_pn_bkgregion_R2($pn_bkg_outer_rad);
		}
		$bkg_area = $pi*$gv->LC_pn_bkgregion_R2*$gv->LC_pn_bkgregion_R2
		    - $pi*$gv->LC_pn_bkgregion_R1*$gv->LC_pn_bkgregion_R1; 
	    } elsif ($bkg_region_shape eq "box") {
		if ($bkg_XL == 0 || $bkg_YL == 0) {
		    print "       PN BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN BKG Region should be defined \n";}
		    &error_code(7);
		}
		$bkg_area = $gv->LC_pn_bkgregion_XLength*$gv->LC_pn_bkgregion_YLength;
	    } else {
		print "       PN BKG Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       PN BKG Region not recognized \n";}
		&error_code(7);
	    }

	    $gv->LC_pn_areafactor($src_area/$bkg_area);

	    if ($gv->LC_pn_srcregion_X == 0 && $gv->LC_pn_srcregion_Y == 0){$pn_lcurve_flag = 0;}      # Something has gone wrong so wont produce spectra

	} elsif ($pn_obsmode eq 'TIMING' || $pn_obsmode eq 'timing') {

	    $gv->LC_pn_srcregion("box");
	    $src_XC = $gv->LC_pn_srcregion_X;
	    $src_YC = $gv->LC_pn_srcregion_Y;
	    $src_XL = $gv->LC_pn_srcregion_XLength;
	    $src_YL = $gv->LC_pn_srcregion_YLength;
	    
	    if (($src_XC == 0 && $src_YC == 0)
		&& ($src_XL == 0 && $src_XL == 0)) {
		$gv->LC_pn_srcregion_X(37.);
		$gv->LC_pn_srcregion_Y(101.);
		$gv->LC_pn_srcregion_XLength(16);
		$gv->LC_pn_srcregion_YLength(199);
	    } 
	    
	    $gv->LC_pn_bkgregion("box");
	    $bkg_XC = $gv->LC_pn_bkgregion_X;
	    $bkg_YC = $gv->LC_pn_bkgregion_Y;
	    $bkg_XL = $gv->LC_pn_bkgregion_XLength;
	    $bkg_YL = $gv->LC_pn_bkgregion_YLength;
	    
	    if (($bkg_XC == 0 && $bkg_YC == 0)
		&& ($bkg_XL == 0 && $bkg_XL == 0)) {
		$gv->LC_pn_bkgregion_X($gv->LC_pn_srcregion_XLength/2.+0.5); # +0.5 to keep it away from the edge of the detector
		$gv->LC_pn_bkgregion_Y($gv->LC_pn_srcregion_Y);
		$gv->LC_pn_bkgregion_XLength($gv->LC_pn_srcregion_XLength);
		$gv->LC_pn_bkgregion_YLength($gv->LC_pn_srcregion_YLength);
	    } 

	    $src_area = $gv->LC_pn_srcregion_XLength*$gv->LC_pn_srcregion_YLength;
	    $bkg_area = $gv->LC_pn_bkgregion_XLength*$gv->LC_pn_bkgregion_YLength;
	    $gv->LC_pn_areafactor($src_area/$bkg_area);

	} else {
	    print "       PN Mode not recognized \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       PN Mode not recognized \n";}
	    &error_code(7);
	}

	print "       PN \n";
	print "         Region SRC  : ",$gv->LC_pn_srcregion,"\n";
	print "         PN SRC XC   : ",$gv->LC_pn_srcregion_X,"\n";
	print "         PN SRC YC   : ",$gv->LC_pn_srcregion_Y,"\n";
	if ($gv->LC_pn_srcregion eq 'circle' || $gv->LC_pn_srcregion eq 'annulus')
	{print "         PN SRC RC   : ",$gv->LC_pn_srcregion_R1," - ",$gv->LC_pn_srcregion_R2,"\n";} 
	if ($gv->LC_pn_srcregion eq 'box' )
	{print "         PN SRC BOX  : ",$gv->LC_pn_srcregion_XLength," - ",$gv->LC_pn_srcregion_YLength,"\n";}
	print "         Region BKG  : ",$gv->LC_pn_bkgregion,"\n";
	print "         PN BKG XC   : ",$gv->LC_pn_bkgregion_X,"\n";
	print "         PN BKG YC   : ",$gv->LC_pn_bkgregion_Y,"\n";
	if ($gv->LC_pn_bkgregion eq 'circle' || $gv->LC_pn_bkgregion eq 'annulus')
	{
	    print "         PN BKG RC   : ",$gv->LC_pn_bkgregion_R1," - ",$gv->LC_pn_bkgregion_R2,"\n";
	    if ($gv->LC_pn_srcregion_R2 > $gv->LC_pn_bkgregion_R1) 
	    {print MYOUTFILE "         WARNING: PN regions : The inner background region radii is smaller than the source region radii \n";} 
	} 
	if ($gv->LC_pn_bkgregion eq 'box' )
	{print "         PN BKG BOX  : ",$gv->LC_pn_bkgregion_XLength," - ",$gv->LC_pn_bkgregion_YLength,"\n";}
	print "         BKG AREA SCALE FACTOR : ",$gv->LC_pn_areafactor,"(Only area taken into account) \n";
	if ($gv->LC_pn_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : PN regions : Maximum allowed radius reached";}
	if ($Log_Flag == 0) {
	    print MYOUTFILE "       PN \n";
	    print MYOUTFILE "         Region SRC  : ",$gv->LC_pn_srcregion,"\n";	
	    print MYOUTFILE "         PN SRC XC   : ",$gv->LC_pn_srcregion_X,"\n";
	    print MYOUTFILE "         PN SRC YC   : ",$gv->LC_pn_srcregion_Y,"\n";
	    if ($gv->LC_pn_srcregion eq 'circle' || $gv->LC_pn_srcregion eq 'annulus')
	    {print MYOUTFILE "         PN SRC RC   : ",$gv->LC_pn_srcregion_R1," - ",$gv->LC_pn_srcregion_R2,"\n";} 
	    if ($gv->LC_pn_srcregion eq 'box' )
	    {print MYOUTFILE "         PN SRC BOX  : ",$gv->LC_pn_srcregion_XLength," - ",$gv->LC_pn_srcregion_YLength,"\n";}
	    print MYOUTFILE "         Region BKG  : ",$gv->LC_pn_bkgregion,"\n";
	    print MYOUTFILE "         PN BKG XC   : ",$gv->LC_pn_bkgregion_X,"\n";
	    print MYOUTFILE "         PN BKG YC   : ",$gv->LC_pn_bkgregion_Y,"\n";
	    if ($gv->LC_pn_bkgregion eq 'circle' || $gv->LC_pn_bkgregion eq 'annulus')
	    {
		print MYOUTFILE "         PN BKG RC   : ",$gv->LC_pn_bkgregion_R1," - ",$gv->LC_pn_bkgregion_R2,"\n";
		if ($gv->LC_pn_srcregion_R2 > $gv->LC_pn_bkgregion_R1) 
		{print MYOUTFILE "         WARNING: PN regions : The inner background region radii is smaller than the source region radii \n";} 
	    } 
	    if ($gv->LC_pn_bkgregion eq 'box' )
	    {print MYOUTFILE "         PN BKG BOX  : ",$gv->LC_pn_bkgregion_XLength," - ",$gv->LC_pn_bkgregion_YLength,"\n";}
	    print MYOUTFILE "         BKG AREA SCALE FACTOR : ",$gv->LC_pn_areafactor," (Only area taken into account)\n";
	    if ($gv->LC_pn_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : PN regions : Maximum allowed radius reached";}
	}
    }
    
    if ($m1_lcurve_flag) {

	if ($m1_obsmode eq 'IMAGING' || $m1_obsmode eq 'imaging') {
#.. Get shape of src and bkg regions:
	    $src_region_shape =  $gv->LC_m1_srcregion;
	    $bkg_region_shape =  $gv->LC_m1_bkgregion;
   
	    ($m1_bore_img_x,$m1_bore_img_y
	     ,$m1_bore_optrad_asec,$m1_bore_optradius
	     ,$m1_bore_img_xbkg,$m1_bore_img_ybkg
	     ,$m1_bkg_inner_rad_asec,$m1_bkg_inner_rad
	     ,$m1_bkg_outer_rad_asec,$m1_bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);
	    ($m1_bore_img_x,$m1_bore_img_y
	     ,$m1_bore_optrad_asec,$m1_bore_optradius
	     ,$m1_bore_img_xbkg,$m1_bore_img_ybkg
	     ,$m1_bkg_inner_rad_asec,$m1_bkg_inner_rad
	     ,$m1_bkg_outer_rad_asec,$m1_bkg_outer_rad)
		=&run_eregionanalyse($m1_gti_image_file,$ra_anal,$dec_anal,"mos1",$m1_obssubmode,$m1_eregionanal_file,"LC"); # MOS1
	 
	    if ($m1_bore_optrad_asec == 300){$m1_lcurve_flag = 0;} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

#.. Check values of parameters defining source and background regions	    
	    $src_XC = $gv->LC_m1_srcregion_X;
	    $src_YC = $gv->LC_m1_srcregion_Y;
	    $src_R1 = $gv->LC_m1_srcregion_R1;
	    $src_R2 = $gv->LC_m1_srcregion_R2;
	    $src_XL = $gv->LC_m1_srcregion_XLength;
	    $src_YL = $gv->LC_m1_srcregion_YLength;
	    $bkg_XC = $gv->LC_m1_bkgregion_X;
	    $bkg_YC = $gv->LC_m1_bkgregion_Y;
	    $bkg_R1 = $gv->LC_m1_bkgregion_R1;
	    $bkg_R2 = $gv->LC_m1_bkgregion_R2;
	    $bkg_XL = $gv->LC_m1_bkgregion_XLength;
	    $bkg_YL = $gv->LC_m1_bkgregion_YLength;

	    if ($src_XC ==0 && $src_YC == 0) {  # if values are set to cero take the default; from eregionanalyse 
		$gv->LC_m1_srcregion_X($m1_bore_img_x);
		$gv->LC_m1_srcregion_Y($m1_bore_img_y);		    
	    }
	    
	    if ($bkg_XC == 0 && $bkg_YC == 0) { # if values are set to cero take the default; from eregionanalyse 
		$gv->LC_m1_bkgregion_X($m1_bore_img_xbkg);
		$gv->LC_m1_bkgregion_Y($m1_bore_img_ybkg);	
	    }

#.. SRC Region	    
	    if ($src_region_shape eq "circle") { 
		$gv->LC_m1_srcregion_R1(0); 
		if ($src_R2 == 0){              # if values are set to cero take the default; from eregionanalyse
		    $gv->LC_m1_srcregion_R2($m1_bore_optradius); 
		}
		$src_area = $pi*$gv->LC_m1_srcregion_R2*$gv->LC_m1_srcregion_R2
		    - $pi*$gv->LC_m1_srcregion_R1*$gv->LC_m1_srcregion_R1;
	    } elsif ($src_region_shape eq "annulus") {
		if ($src_R1 == 0 || $src_R2 == 0) {
		    print "       MOS1 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 SRC Region should be defined \n";}
		    &error_code(7);
		}
		$src_area = $pi*$gv->LC_m1_srcregion_R2*$gv->LC_m1_srcregion_R2
		    - $pi*$gv->LC_m1_srcregion_R1*$gv->LC_m1_srcregion_R1;
	    } elsif ($src_region_shape eq "box") {
		if ($src_XL == 0 || $src_YL == 0) {
		    print "       MOS1 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 SRC Region should be defined \n";}
		    &error_code(7);
		}
		$src_area = $gv->LC_m1_srcregion_XLength*$gv->LC_m1_srcregion_YLength;
	    } else {
		print "       MOS1 SRC Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 SRC Region not recognized \n";}
		&error_code(7);
	    }
	    
#.. BKG Region	    
	    if ($bkg_region_shape eq "circle") { 		
		$gv->LC_m1_bkgregion_R1(0);
		if ($bkg_R2 == 0){ 
		    print "       MOS1 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 BKG Region should be defined \n";}
		    &error_code(7);
		}
		$bkg_area = $pi*$gv->LC_m1_bkgregion_R2*$gv->LC_m1_bkgregion_R2
		    - $pi*$gv->LC_m1_bkgregion_R1*$gv->LC_m1_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "annulus") {
		if ($bkg_R1 == 0 || $bkg_R2 == 0) {      # if values are set to cero take the default; from eregionanalyse
		    $gv->LC_m1_bkgregion_R1($m1_bkg_inner_rad);
		    $gv->LC_m1_bkgregion_R2($m1_bkg_outer_rad);
		}
		$bkg_area = $pi*$gv->LC_m1_bkgregion_R2*$gv->LC_m1_bkgregion_R2
		    - $pi*$gv->LC_m1_bkgregion_R1*$gv->LC_m1_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "box") {
		if ($bkg_XL == 0 || $bkg_YL == 0) {
		    print "       MOS1 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 BKG Region should be defined \n";}
		    &error_code(7);
		}
		$bkg_area = $gv->LC_m1_bkgregion_XLength*$gv->LC_m1_bkgregion_YLength;
	    } else {
		print "       MOS1 BKG Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 BKG Region not recognized \n";}
		&error_code(7);
	    }
	    
	    $gv->LC_m1_areafactor($src_area/$bkg_area);
	   
	    if ($gv->LC_m1_srcregion_X == 0 && $gv->LC_m1_srcregion_Y == 0){$m1_lcurve_flag = 0;}      # Something has gone wrong so wont produce spectra
	    
	} elsif ($m1_obsmode eq 'TIMING' || $m1_obsmode eq 'timing') {
	    
	    $gv->LC_m1_srcregion("box");
	    $src_XC = $gv->LC_m1_srcregion_X;
	    $src_YC = $gv->LC_m1_srcregion_Y;
	    $src_XL = $gv->LC_m1_srcregion_XLength;
	    $src_YL = $gv->LC_m1_srcregion_YLength;
	    if (($src_XC == 0 && $src_YC == 0)
		&& ($src_XL == 0 && $src_XL == 0)) {
		$t_start=$obsid_instruments{$source_obsID}{'mos1'}{'starting_time'};
		$t_end=$obsid_instruments{$source_obsID}{'mos1'}{'ending_time'};
		$interval=$t_end-$t_start;
		$mid_interval=$t_start+$interval/2.;		
		$gv->LC_m1_srcregion_X(299.5);
		$gv->LC_m1_srcregion_Y($mid_interval);
		$gv->LC_m1_srcregion_XLength(20);
		$gv->LC_m1_srcregion_YLength($interval);
	    } 
	    
	    $gv->LC_m1_bkgregion("box");
	    $bkg_XC = $gv->LC_m1_bkgregion_X;
	    $bkg_YC = $gv->LC_m1_bkgregion_Y;
	    $bkg_XL = $gv->LC_m1_bkgregion_XLength;
	    $bkg_YL = $gv->LC_m1_bkgregion_YLength;
	    
	    if (($bkg_XC == 0 && $bkg_YC == 0)
		&& ($bkg_XL == 0 && $bkg_XL == 0)) {
		$gv->LC_m1_bkgregion_X(256.5); # lowest column value in MOS Timing 131
		$gv->LC_m1_bkgregion_Y($gv->LC_m1_srcregion_Y);
		$gv->LC_m1_bkgregion_XLength($gv->LC_m1_srcregion_XLength);
		$gv->LC_m1_bkgregion_YLength($gv->LC_m1_srcregion_YLength);
	    }

	    $src_area = $gv->LC_m1_srcregion_XLength*$gv->LC_m1_srcregion_YLength;
	    $bkg_area = $gv->LC_m1_bkgregion_XLength*$gv->LC_m1_bkgregion_YLength;
	    $gv->LC_m1_areafactor($src_area/$bkg_area);

	} else {
	    print "       MOS1 Mode not recognized \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 Mode not recognized \n";}
	    &error_code(7);
	}

	print "       MOS1 \n";
	print "         Region SRC  : ",$gv->LC_m1_srcregion,"\n";
	print "         MOS1 SRC XC   : ",$gv->LC_m1_srcregion_X,"\n";
	print "         MOS1 SRC YC   : ",$gv->LC_m1_srcregion_Y,"\n";
	if ($gv->LC_m1_srcregion eq 'circle' || $gv->LC_m1_srcregion eq 'annulus')
	{print "         MOS1 SRC RC   : ",$gv->LC_m1_srcregion_R1," - ",$gv->LC_m1_srcregion_R2,"\n";} 
	if ($gv->LC_m1_srcregion eq 'box' )
	{print "         MOS1 SRC BOX  : ",$gv->LC_m1_srcregion_XLength," - ",$gv->LC_m1_srcregion_YLength,"\n";}
	print "         Region BKG  : ",$gv->LC_m1_bkgregion,"\n";
	print "         MOS1 BKG XC   : ",$gv->LC_m1_bkgregion_X,"\n";
	print "         MOS1 BKG YC   : ",$gv->LC_m1_bkgregion_Y,"\n";	   
	if ($gv->LC_m1_bkgregion eq 'circle' || $gv->LC_m1_bkgregion eq 'annulus')
	{
	    print "         MOS1 BKG RC   : ",$gv->LC_m1_bkgregion_R1," - ",$gv->LC_m1_bkgregion_R2,"\n";
	    if ($gv->LC_m1_srcregion_R2 > $gv->LC_m1_bkgregion_R1) 
	    {print "         WARNING: MOS1 regions : The inner background region radii is smaller than the source region radii \n";} 
	} 
	if ($gv->LC_m1_bkgregion eq 'box' )
	{print "         MOS1 BKG BOX  : ",$gv->LC_m1_bkgregion_XLength," - ",$gv->LC_m1_bkgregion_YLength,"\n";}
	print "         BKG AREA SCALE FACTOR : ",$gv->LC_m1_areafactor," (Only area taken into account)\n";
	if ($gv->LC_m1_srcregion_R2 == 6000){print "         WARNING : MOS1 regions : Maximum allowed radius reached";}	
	if ($Log_Flag == 0) {
	    print MYOUTFILE "       MOS1 \n";
	    print MYOUTFILE "         Region SRC  : ",$gv->LC_m1_srcregion,"\n";
	    print MYOUTFILE "         MOS1 SRC XC   : ",$gv->LC_m1_srcregion_X,"\n";
	    print MYOUTFILE "         MOS1 SRC YC   : ",$gv->LC_m1_srcregion_Y,"\n";
	    if ($gv->LC_m1_srcregion eq 'circle' || $gv->LC_m1_srcregion eq 'annulus')
	    {print MYOUTFILE "         MOS1 SRC RC   : ",$gv->LC_m1_srcregion_R1," - ",$gv->LC_m1_srcregion_R2,"\n";} 
	    if ($gv->LC_m1_srcregion eq 'box' )
	    {print MYOUTFILE "         MOS1 SRC BOX  : ",$gv->LC_m1_srcregion_XLength," - ",$gv->LC_m1_srcregion_YLength,"\n";}
	    print MYOUTFILE "         Region BKG  : ",$gv->LC_m1_bkgregion,"\n";
	    print MYOUTFILE "         MOS1 BKG XC   : ",$gv->LC_m1_bkgregion_X,"\n";
	    print MYOUTFILE "         MOS1 BKG YC   : ",$gv->LC_m1_bkgregion_Y,"\n";
	    if ($gv->LC_m1_bkgregion eq 'circle' || $gv->LC_m1_bkgregion eq 'annulus')
	    {
		print MYOUTFILE "         MOS1 BKG RC   : ",$gv->LC_m1_bkgregion_R1," - ",$gv->LC_m1_bkgregion_R2,"\n";
		if ($gv->LC_m1_srcregion_R2 > $gv->LC_m1_bkgregion_R1) 
		{print MYOUTFILE "         WARNING: MOS1 regions : The inner background region radii is smaller than the source region radii \n";} 
	    } 
	    if ($gv->LC_m1_bkgregion eq 'box' )
	    {print MYOUTFILE "         MOS1 BKG BOX  : ",$gv->LC_m1_bkgregion_XLength," - ",$gv->LC_m1_bkgregion_YLength,"\n";}
	    print MYOUTFILE "         BKG AREA SCALE FACTOR : ",$gv->LC_m1_areafactor," (Only area taken into account)\n";
	    if ($gv->LC_m1_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : MOS1 regions : Maximum allowed radius reached";}
	}
    }

    if ($m2_lcurve_flag) { 

	if ($m2_obsmode eq 'IMAGING' || $m2_obsmode eq 'imaging') {
#.. Get shape of src and bkg regions:
	    $src_region_shape =  $gv->LC_m2_srcregion;
	    $bkg_region_shape =  $gv->LC_m2_bkgregion;
	    
	    ($m2_bore_img_x,$m2_bore_img_y
	     ,$m2_bore_optrad_asec,$m2_bore_optradius
	     ,$m2_bore_img_xbkg,$m2_bore_img_ybkg
	     ,$m2_bkg_inner_rad_asec,$m2_bkg_inner_rad
	     ,$m2_bkg_outer_rad_asec,$m2_bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);	    
	    ($m2_bore_img_x,$m2_bore_img_y
	     ,$m2_bore_optrad_asec,$m2_bore_optradius
	     ,$m2_bore_img_xbkg,$m2_bore_img_ybkg
	     ,$m2_bkg_inner_rad_asec,$m2_bkg_inner_rad
	     ,$m2_bkg_outer_rad_asec,$m2_bkg_outer_rad)
		=&run_eregionanalyse($m2_gti_image_file,$ra_anal,$dec_anal,"mos2",$m2_obssubmode,$m2_eregionanal_file,"LC"); # MOS2 

	    if ($m2_bore_optrad_asec == 300){$m2_lcurve_flag = 0;} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

#.. Check values of parameters defining source and background regions	    	    
	    $src_XC = $gv->LC_m2_srcregion_X;
	    $src_YC = $gv->LC_m2_srcregion_Y;
	    $src_R1 = $gv->LC_m2_srcregion_R1;
	    $src_R2 = $gv->LC_m2_srcregion_R2;
	    $src_XL = $gv->LC_m2_srcregion_XLength;
	    $src_YL = $gv->LC_m2_srcregion_YLength;
	    $bkg_XC = $gv->LC_m2_bkgregion_X;
	    $bkg_YC = $gv->LC_m2_bkgregion_Y;
	    $bkg_R1 = $gv->LC_m2_bkgregion_R1;
	    $bkg_R2 = $gv->LC_m2_bkgregion_R2;
	    $bkg_XL = $gv->LC_m2_bkgregion_XLength;
	    $bkg_YL = $gv->LC_m2_bkgregion_YLength;

	    if ($src_XC ==0 && $src_YC == 0) {  # if values are set to cero take the default; from eregionanalyse 
		$gv->LC_m2_srcregion_X($m2_bore_img_x);
		$gv->LC_m2_srcregion_Y($m2_bore_img_y);		    
	    }
	    
	    if ($bkg_XC == 0 && $bkg_YC == 0) { # if values are set to cero take the default; from eregionanalyse 
		$gv->LC_m2_bkgregion_X($m2_bore_img_xbkg);
		$gv->LC_m2_bkgregion_Y($m2_bore_img_ybkg);	
	    }

#.. SRC Region	    
	    if ($src_region_shape eq "circle") { 
		$gv->LC_m2_srcregion_R1(0); 
		if ($src_R2 == 0){              # if values are set to cero take the default; from eregionanalyse
		    $gv->LC_m2_srcregion_R2($m2_bore_optradius); 
		}
		$src_area = $pi*$gv->LC_m2_srcregion_R2*$gv->LC_m2_srcregion_R2
		    - $pi*$gv->LC_m2_srcregion_R1*$gv->LC_m2_srcregion_R1;
	    } elsif ($src_region_shape eq "annulus") {
		if ($src_R1 == 0 || $src_R2 == 0) {
		    print "       MOS2 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 SRC Region should be defined \n";}
		    &error_code(7);
		}
		$src_area = $pi*$gv->LC_m2_srcregion_R2*$gv->LC_m2_srcregion_R2
		    - $pi*$gv->LC_m2_srcregion_R1*$gv->LC_m2_srcregion_R1;
	    } elsif ($src_region_shape eq "box") {
		if ($src_XL == 0 || $src_YL == 0) {
		    print "       MOS2 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 SRC Region should be defined \n";}
		    &error_code(7);
		}
		$src_area = $gv->LC_m2_srcregion_XLength*$gv->LC_m2_srcregion_YLength;
	    } else {
		print "       MOS2 SRC Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 SRC Region not recognized \n";}
		&error_code(7);
	    }
	    
#.. BKG Region	    
	    if ($bkg_region_shape eq "circle") { 
		$gv->LC_m2_bkgregion_R1(0);
		if ($bkg_R2 == 0){ 
		    print "       MOS2 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 BKG Region should be defined \n";}
		    &error_code(7);
		}
		$bkg_area = $pi*$gv->LC_m2_bkgregion_R2*$gv->LC_m2_bkgregion_R2
		    - $pi*$gv->LC_m2_bkgregion_R1*$gv->LC_m2_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "annulus") {	
		if ($bkg_R1 == 0 || $bkg_R2 == 0) {      # if values are set to cero take the default; from eregionanalyse
		    $gv->LC_m2_bkgregion_R1($m2_bkg_inner_rad);
		    $gv->LC_m2_bkgregion_R2($m2_bkg_outer_rad);
		}
		$bkg_area = $pi*$gv->LC_m2_bkgregion_R2*$gv->LC_m2_bkgregion_R2
		    - $pi*$gv->LC_m2_bkgregion_R1*$gv->LC_m2_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "box") {
		if ($bkg_XL == 0 || $bkg_YL == 0) {
		    print "       MOS2 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 BKG Region should be defined \n";}
		    &error_code(7);
		}
		$bkg_area = $gv->LC_m2_bkgregion_XLength*$gv->LC_m2_bkgregion_YLength;
	    } else {
		print "       MOS2 BKG Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 BKG Region not recognized \n";}
		&error_code(7);
	    }	    

	    $gv->LC_m2_areafactor($src_area/$bkg_area);

	    if ($gv->LC_m2_srcregion_X == 0 && $gv->LC_m2_srcregion_Y){$m2_lcurve_flag = 0;}      # Something has gone wrong so wont produce spectra
	    
	} elsif ($m2_obsmode eq 'TIMING' || $m2_obsmode eq 'timing') {
	    
	    $gv->LC_m2_srcregion("box");
	    $src_XC = $gv->LC_m2_srcregion_X;
	    $src_YC = $gv->LC_m2_srcregion_Y;
	    $src_XL = $gv->LC_m2_srcregion_XLength;
	    $src_YL = $gv->LC_m2_srcregion_YLength;
	    
	    if (($src_XC == 0 && $src_YC == 0)
		&& ($src_XL == 0 && $src_XL == 0)) {
		$t_start=$obsid_instruments{$source_obsID}{'mos2'}{'starting_time'};
		$t_end=$obsid_instruments{$source_obsID}{'mos2'}{'ending_time'};
		$interval=$t_end-$t_start;
		$mid_interval=$t_start+$interval/2.;
		$gv->LC_m2_srcregion_X(299.5);
		$gv->LC_m2_srcregion_Y($mid_interval);
		$gv->LC_m2_srcregion_XLength(20);
		$gv->LC_m2_srcregion_YLength($interval);
	    } 
	    
	    $gv->LC_m2_bkgregion("box");
	    $bkg_XC = $gv->LC_m2_bkgregion_X;
	    $bkg_YC = $gv->LC_m2_bkgregion_Y;
	    $bkg_XL = $gv->LC_m2_bkgregion_XLength;
	    $bkg_YL = $gv->LC_m2_bkgregion_YLength;

	    if (($bkg_XC == 0 && $bkg_YC == 0)
		&& ($bkg_XL == 0 && $bkg_XL == 0)) {
		$gv->LC_m2_bkgregion_X(259.); # 256.5 lowest column in MOS timing
		$gv->LC_m2_bkgregion_Y($gv->LC_m2_srcregion_Y);
		$gv->LC_m2_bkgregion_XLength($gv->LC_m2_srcregion_XLength);
		$gv->LC_m2_bkgregion_YLength($gv->LC_m2_srcregion_YLength);
	    }
	    
	    $src_area = $gv->LC_m2_srcregion_XLength*$gv->LC_m2_srcregion_YLength;
	    $bkg_area = $gv->LC_m2_bkgregion_XLength*$gv->LC_m2_bkgregion_YLength;
	    $gv->LC_m2_areafactor($src_area/$bkg_area);

	} else {
	    print "       MOS2 Mode not recognized \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 Mode not recognized \n";}
	    &error_code(7);
	}
	
	print "       MOS2 \n";
	print "         Region SRC  : ",$gv->LC_m2_srcregion,"\n";
	print "         MOS2 SRC XC   : ",$gv->LC_m2_srcregion_X,"\n";
	print "         MOS2 SRC YC   : ",$gv->LC_m2_srcregion_Y,"\n";
	if ($gv->LC_m2_srcregion eq 'circle' || $gv->LC_m2_srcregion eq 'annulus')
	{print "         MOS2 SRC RC   : ",$gv->LC_m2_srcregion_R1," - ",$gv->LC_m2_srcregion_R2,"\n";} 
	if ($gv->LC_m2_srcregion eq 'box' )
	{print "         MOS2 SRC BOX  : ",$gv->LC_m2_srcregion_XLength," - ",$gv->LC_m2_srcregion_YLength,"\n";}
	print "         Region BKG  : ",$gv->LC_m2_bkgregion,"\n";
	print "         MOS2 BKG XC   : ",$gv->LC_m2_bkgregion_X,"\n";
	print "         MOS2 BKG YC   : ",$gv->LC_m2_bkgregion_Y,"\n";	   
	if ($gv->LC_m2_bkgregion eq 'circle' || $gv->LC_m2_bkgregion eq 'annulus')
	{
	    print "         MOS2 BKG RC   : ",$gv->LC_m2_bkgregion_R1," - ",$gv->LC_m2_bkgregion_R2,"\n";
	    if ($gv->LC_m2_srcregion_R2 > $gv->LC_m2_bkgregion_R1) 
	    {print "         WARNING: MOS2 regions : The inner background region radii is smaller than the source region radii \n";} 
	} 
	if ($gv->LC_m2_bkgregion eq 'box' )
	{print "         MOS2 BKG BOX  : ",$gv->LC_m2_bkgregion_XLength," - ",$gv->LC_m2_bkgregion_YLength,"\n";}
	print "         BKG AREA SCALE FACTOR : ",$gv->LC_m2_areafactor," (Only area taken into account)\n";
	if ($gv->LC_m2_srcregion_R2 == 6000){print "         WARNING : MOS2 regions : Maximum allowed radius reached";}
	if ($Log_Flag == 0) {
	    print MYOUTFILE "       MOS2 \n";
	    print MYOUTFILE "         Region SRC    : ",$gv->LC_m2_srcregion,"\n";
	    print MYOUTFILE "         MOS2 SRC XC   : ",$gv->LC_m2_srcregion_X,"\n";
	    print MYOUTFILE "         MOS2 SRC YC   : ",$gv->LC_m2_srcregion_Y,"\n";
	    if ($gv->LC_m2_srcregion eq 'circle' || $gv->LC_m2_srcregion eq 'annulus')
	    {print MYOUTFILE "         MOS2 SRC RC   : ",$gv->LC_m2_srcregion_R1," - ",$gv->LC_m2_srcregion_R2,"\n";} 
	    if ($gv->LC_m2_srcregion eq 'box' )
	    {print MYOUTFILE "         MOS2 SRC BOX  : ",$gv->LC_m2_srcregion_XLength," - ",$gv->LC_m2_srcregion_YLength,"\n";}
	    print MYOUTFILE "         Region BKG    : ",$gv->LC_m2_bkgregion,"\n";
	    print MYOUTFILE "         MOS2 BKG XC   : ",$gv->LC_m2_bkgregion_X,"\n";
	    print MYOUTFILE "         MOS2 BKG YC   : ",$gv->LC_m2_bkgregion_Y,"\n";
	    if ($gv->LC_m2_bkgregion eq 'circle' || $gv->LC_m2_bkgregion eq 'annulus')
	    {
		print MYOUTFILE "         MOS2 BKG RC   : ",$gv->LC_m2_bkgregion_R1," - ",$gv->LC_m2_bkgregion_R2,"\n";
		if ($gv->LC_m2_srcregion_R2 > $gv->LC_m2_bkgregion_R1) 
		{print MYOUTFILE "         WARNING: MOS2 regions : The inner background region radii is smaller than the source region radii \n";} 
	    } 
	    if ($gv->LC_m2_bkgregion eq 'box' )	
	    {print MYOUTFILE "         MOS2 BKG BOX  : ",$gv->LC_m2_bkgregion_XLength," - ",$gv->LC_m2_bkgregion_YLength,"\n";}
	    print MYOUTFILE "         BKG AREA SCALE FACTOR : ",$gv->LC_m2_areafactor," (Only area taken into account)\n";
	    if ($gv->LC_m2_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : MOS2 regions : Maximum allowed radius reached";}
	}
    }

#.. run epatplot
	if ($gv->ANA_epatplot)
	{
    	if ($pn_spectral_flag) {&run_epatplot($pn_gti_cleanevents_file,"pn","SPC",$pn_obsmode);}   # PN
	    if ($m1_spectral_flag) {&run_epatplot($m1_gti_cleanevents_file,"m1","SPC",$m1_obsmode);} # MOS1
    	if ($m2_spectral_flag) {&run_epatplot($m2_gti_cleanevents_file,"m2","SPC",$m2_obsmode);} # MOS2
    	# Set the flag to false. If run_all has been selected, produce_lightcurve will not run againg epatplot.
    	$gv->ANA_epatplot(0);
	}



#.. Prepare light curve log file and output information
    if ($Log_Lcur_Flag == 0) {
	my $lcurve_log=$results_directory."lcurve_".$source_name."\.log";
	if (-e $lcurve_log) {unlink("$lcurve_log") or die "Cannot rm file: $!";}    
	open(LCURVELOG, "> $lcurve_log") or die "Cant open output file: $! \n";    #open for write, overwrite
	
	print LCURVELOG "   \#> Centroid of source to analyze (Physical Units) \n";
	
	print LCURVELOG "       PN \n";
	print LCURVELOG "         SRC Region : ",$gv->LC_pn_srcregion,"\n";
	print LCURVELOG "         SRC XC     : ",$gv->LC_pn_srcregion_X," \n";
	print LCURVELOG "         SRC YC     : ",$gv->LC_pn_srcregion_Y," \n";
	if ($gv->LC_pn_srcregion eq "circle" || $gv->LC_pn_srcregion eq "annulus")
	{print LCURVELOG "         SRC RADII  : ",$gv->LC_pn_srcregion_R1," - ",$gv->LC_pn_srcregion_R2," \n";}
	if ($gv->LC_pn_srcregion eq "box") 
	{print LCURVELOG "         SRC BOX    : ",$gv->LC_pn_srcregion_XLength," - ",$gv->LC_pn_srcregion_YLength," \n";}
	print LCURVELOG "         BKG Region : ",$gv->LC_pn_bkgregion,"\n";
	print LCURVELOG "         BKG XC     : ",$gv->LC_pn_bkgregion_X," \n";
	print LCURVELOG "         BKG YC     : ",$gv->LC_pn_bkgregion_Y," \n";
	if ($gv->LC_pn_srcregion eq "circle" || $gv->LC_pn_srcregion eq "annulus") 
	{
	    print LCURVELOG "         BKG RADII  : ",$gv->LC_pn_bkgregion_R1," - ",$gv->LC_pn_bkgregion_R2," \n";
	    if ($gv->LC_pn_srcregion_R2 > $gv->LC_pn_bkgregion_R1) 
	    {print LCURVELOG "         WARNING: PN regions : The inner background region radii is smaller than the source region radii \n";} 
	}
	if ($gv->LC_pn_srcregion eq "box") 
	{print LCURVELOG "         BKG BOX    : ",$gv->LC_pn_bkgregion_XLength," - ",$gv->LC_pn_bkgregion_YLength," \n";}
	print LCURVELOG "         BKG AREA SCALE FACTOR : ",$gv->LC_pn_areafactor," (Only area taken into account)\n";
	if ($gv->LC_pn_srcregion_R2 == 6000){print LCURVELOG  "         WARNING : PN regions : Maximum allowed radius reached";}
	
	print LCURVELOG "       MOS1 \n";
	print LCURVELOG "         SRC Region : ",$gv->LC_m1_srcregion,"\n";
	print LCURVELOG "         SRC XC     : ",$gv->LC_m1_srcregion_X," \n";
	print LCURVELOG "         SRC YC     : ",$gv->LC_m1_srcregion_Y," \n";
	if ($gv->LC_m1_srcregion eq "circle" || $gv->LC_m1_srcregion eq "annulus")
	{print LCURVELOG "         SRC RADII  : ",$gv->LC_m1_srcregion_R1," - ",$gv->LC_m1_srcregion_R2," \n";}
	if ($gv->LC_m1_srcregion eq "box") 
	{print LCURVELOG "         SRC BOX    : ", $gv->LC_m1_srcregion_XLength," - ",$gv->LC_m1_srcregion_YLength," \n";}
	print LCURVELOG "         BKG Region : ",$gv->LC_m1_bkgregion,"\n";
	print LCURVELOG "         BKG XC     : ",$gv->LC_m1_bkgregion_X," \n";
	print LCURVELOG "         BKG YC     : ",$gv->LC_m1_bkgregion_Y," \n";
	if ($gv->LC_m1_srcregion eq "circle" || $gv->LC_m1_srcregion eq "annulus") 
	{
	    print LCURVELOG "         BKG RADII  : ",$gv->LC_m1_bkgregion_R1," - ",$gv->LC_m1_bkgregion_R2," \n";
	    if ($gv->LC_m1_srcregion_R2 > $gv->LC_m1_bkgregion_R1) 
	    {print LCURVELOG "         WARNING: MOS1 regions : The inner background region radii is smaller than the source region radii \n";}
	}
	if ($gv->LC_m1_srcregion eq "box") 
	{print LCURVELOG "         BKG BOX    : ",$gv->LC_m1_bkgregion_XLength," - ",$gv->LC_m1_bkgregion_YLength," \n";}
	print LCURVELOG "         BKG AREA SCALE FACTOR : ",$gv->LC_m1_areafactor," (Only area taken into account)\n";
	if ($gv->LC_m1_srcregion_R2 == 6000){print LCURVELOG  "         WARNING : MOS1 regions : Maximum allowed radius reached";}
	 
	print LCURVELOG "       MOS2 \n";
	print LCURVELOG "         SRC Region : ",$gv->LC_m2_srcregion,"\n";
	print LCURVELOG "         SRC XC     : ",$gv->LC_m2_srcregion_X," \n";
	print LCURVELOG "         SRC YC     : ",$gv->LC_m2_srcregion_Y," \n";
	if ($gv->LC_m2_srcregion eq "circle" || $gv->LC_m2_srcregion eq "annulus")
	{print LCURVELOG "         SRC RADII  : ",$gv->LC_m2_srcregion_R1," - ",$gv->LC_m2_srcregion_R2," \n";}
	if ($gv->LC_m2_srcregion eq "box") 
	{print LCURVELOG "         SRC BOX    : ", $gv->LC_m2_srcregion_XLength," - ",$gv->LC_m2_srcregion_YLength," \n";}
	print LCURVELOG "         BKG Region : ",$gv->LC_m2_bkgregion,"\n";
	print LCURVELOG "         BKG XC     : ",$gv->LC_m2_bkgregion_X," \n";
	print LCURVELOG "         BKG YC     : ",$gv->LC_m2_bkgregion_Y," \n";
	if ($gv->LC_m2_srcregion eq "circle" || $gv->LC_m2_srcregion eq "annulus") 
	{
	    print LCURVELOG "         BKG RADII  : ",$gv->LC_m2_bkgregion_R1," - ",$gv->LC_m2_bkgregion_R2," \n";
	    if ($gv->LC_m2_srcregion_R2 > $gv->LC_m2_bkgregion_R1) 
	    {print LCURVELOG "         WARNING: MOS2 regions : The inner background region radii is smaller than the source region radii \n";}
	}
	if ($gv->LC_m2_srcregion eq "box") 
	{print LCURVELOG "         BKG BOX    : ",$gv->LC_m2_bkgregion_XLength," - ",$gv->LC_m2_bkgregion_YLength," \n";}
	print LCURVELOG "         BKG AREA SCALE FACTOR : ",$gv->LC_m2_areafactor," (Only area taken into account)\n";	
	if ($gv->LC_m2_srcregion_R2 == 6000){print LCURVELOG  "         WARNING : MOS2 regions : Maximum allowed radius reached";}
    }	
    
    if (!$pn_lcurve_flag && !$m1_lcurve_flag && !$m2_lcurve_flag) {
	print "   \#> No information to produce light curve in any camera .... exiting \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> No information to produce light curve in any camera .... exiting \n";}
	if ($Log_Lcur_Flag == 0) {print LCURVELOG "   \#> No information to produce light curve in any camera .... exiting \n";}
	if ($Log_Lcur_Flag == 0) {close(LCURVELOG);}
	
	&error_code(0); 
	return(0);
    }
    
    chdir($lcurve_directory);

#.. Make image plots and create region files used to extract the Light curves
    print "\n   \#> Producing plots and region files ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Producing plots and region files ..... \n";}

    if (-e $pn_lcurve_region_file) {unlink("$pn_lcurve_region_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m1_lcurve_region_file) {unlink("$m1_lcurve_region_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m2_lcurve_region_file) {unlink("$m2_lcurve_region_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $pn_lcurve_bkgregion_file) {unlink("$pn_lcurve_bkgregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m1_lcurve_bkgregion_file) {unlink("$m1_lcurve_bkgregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m2_lcurve_bkgregion_file) {unlink("$m2_lcurve_bkgregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $pn_lcurve_combinedregion_file) {unlink("$pn_lcurve_combinedregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m1_lcurve_combinedregion_file) {unlink("$m1_lcurve_combinedregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m2_lcurve_combinedregion_file) {unlink("$m2_lcurve_combinedregion_file") or die "Cannot rm file $temp_file: $!";}

    if ($gv->LC_pn_srcregion eq "circle" || $gv->LC_pn_srcregion eq "annulus") 
    {$x= $gv->LC_pn_srcregion_X; $y= $gv->LC_pn_srcregion_Y; $L1=$gv->LC_pn_srcregion_R1; $L2 =$gv->LC_pn_srcregion_R2;}
    if ($gv->LC_pn_srcregion eq "box") 
    {$x= $gv->LC_pn_srcregion_X; $y= $gv->LC_pn_srcregion_Y; $L1=$gv->LC_pn_srcregion_XLength; $L2 =$gv->LC_pn_srcregion_YLength;}
    if ($pn_lcurve_flag) {&addsourcetoregionfile($pn_lcurve_region_file,$x,$y,$L1,$L2,"yellow","",$gv->LC_pn_srcregion,"physical");} # PN
    if ($pn_lcurve_flag) {&addsourcetoregionfile($pn_lcurve_combinedregion_file,$x,$y,$L1,$L2,"yellow","",$gv->LC_pn_srcregion,"physical");} # PN

    if ($gv->LC_m1_srcregion eq "circle" || $gv->LC_m1_srcregion eq "annulus") 
    {$x= $gv->LC_m1_srcregion_X; $y= $gv->LC_m1_srcregion_Y; $L1=$gv->LC_m1_srcregion_R1; $L2 =$gv->LC_m1_srcregion_R2;}
    if ($gv->LC_m1_srcregion eq "box") 
    {$x= $gv->LC_m1_srcregion_X; $y= $gv->LC_m1_srcregion_Y; $L1=$gv->LC_m1_srcregion_XLength; $L2 =$gv->LC_m1_srcregion_YLength;}				     
    if ($m1_lcurve_flag) {&addsourcetoregionfile($m1_lcurve_region_file,$x,$y,$L1,$L2,"yellow","",$gv->LC_m1_srcregion,"physical");}   # MOS1 
    if ($m1_lcurve_flag) {&addsourcetoregionfile($m1_lcurve_combinedregion_file,$x,$y,$L1,$L2,"yellow","",$gv->LC_m1_srcregion,"physical");}   # MOS1 

    if ($gv->LC_m2_srcregion eq "circle" || $gv->LC_m2_srcregion eq "annulus") 
    {$x= $gv->LC_m2_srcregion_X; $y= $gv->LC_m2_srcregion_Y; $L1=$gv->LC_m2_srcregion_R1; $L2 =$gv->LC_m2_srcregion_R2;}
    if ($gv->LC_m2_srcregion eq "box") 
    {$x= $gv->LC_m2_srcregion_X; $y= $gv->LC_m2_srcregion_Y; $L1=$gv->LC_m2_srcregion_XLength; $L2 =$gv->LC_m2_srcregion_YLength;}					     
    if ($m2_lcurve_flag) {&addsourcetoregionfile($m2_lcurve_region_file,$x,$y,$L1,$L2,"yellow","",$gv->LC_m2_srcregion,"physical");}   # MOS2
    if ($m2_lcurve_flag) {&addsourcetoregionfile($m2_lcurve_combinedregion_file,$x,$y,$L1,$L2,"yellow","",$gv->LC_m2_srcregion,"physical");}   # MOS2

    if ($gv->LC_pn_bkgregion eq "circle" || $gv->LC_pn_bkgregion eq "annulus") 
    {$x= $gv->LC_pn_bkgregion_X; $y= $gv->LC_pn_bkgregion_Y; $L1=$gv->LC_pn_bkgregion_R1; $L2 =$gv->LC_pn_bkgregion_R2;}
    if ($gv->LC_pn_bkgregion eq "box") 
    {$x= $gv->LC_pn_bkgregion_X; $y= $gv->LC_pn_bkgregion_Y; $L1=$gv->LC_pn_bkgregion_XLength; $L2 =$gv->LC_pn_bkgregion_YLength;}
    if ($pn_lcurve_flag) {&addsourcetoregionfile($pn_lcurve_bkgregion_file,$x,$y,$L1,$L2,"red","",$gv->LC_pn_bkgregion,"physical");} # PN
    if ($pn_lcurve_flag) {&addsourcetoregionfile($pn_lcurve_combinedregion_file,$x,$y,$L1,$L2,"red","",$gv->LC_pn_bkgregion,"physical");} # PN

    if ($gv->LC_m1_bkgregion eq "circle" || $gv->LC_m1_bkgregion eq "annulus") 
    {$x= $gv->LC_m1_bkgregion_X; $y= $gv->LC_m1_bkgregion_Y; $L1=$gv->LC_m1_bkgregion_R1; $L2 =$gv->LC_m1_bkgregion_R2;}
    if ($gv->LC_m1_bkgregion eq "box") 
    {$x= $gv->LC_m1_bkgregion_X; $y= $gv->LC_m1_bkgregion_Y; $L1=$gv->LC_m1_bkgregion_XLength; $L2 =$gv->LC_m1_bkgregion_YLength;}
    if ($m1_lcurve_flag) {&addsourcetoregionfile($m1_lcurve_bkgregion_file,$x,$y,$L1,$L2,"red","",$gv->LC_m1_bkgregion,"physical");} # MOS1   
    if ($m1_lcurve_flag) {&addsourcetoregionfile($m1_lcurve_combinedregion_file,$x,$y,$L1,$L2,"red","",$gv->LC_m1_bkgregion,"physical");} # MOS1   

    if ($gv->LC_m2_bkgregion eq "circle" || $gv->LC_m2_bkgregion eq "annulus") 
    {$x= $gv->LC_m2_bkgregion_X; $y= $gv->LC_m2_bkgregion_Y; $L1=$gv->LC_m2_bkgregion_R1; $L2 =$gv->LC_m2_bkgregion_R2;}
    if ($gv->LC_m2_bkgregion eq "box") 
    {$x= $gv->LC_m2_bkgregion_X; $y= $gv->LC_m2_bkgregion_Y; $L1=$gv->LC_m2_bkgregion_XLength; $L2 =$gv->LC_m2_bkgregion_YLength;}
    if ($m2_lcurve_flag) {&addsourcetoregionfile($m2_lcurve_bkgregion_file,$x,$y,$L1,$L2,"red","",$gv->LC_m2_bkgregion,"physical");} # MOS2
    if ($m2_lcurve_flag) {&addsourcetoregionfile($m2_lcurve_combinedregion_file,$x,$y,$L1,$L2,"red","",$gv->LC_m2_bkgregion,"physical");} # MOS2   				

    print "\n   \#> Producing Source and BKG Light Curve files ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Producing Source and BKG Light Curve files ..... \n";}

    if ($pn_lcurve_flag){
		$SRC_XC    = $gv->LC_pn_srcregion_X;
		$SRC_YC    = $gv->LC_pn_srcregion_Y;
		if ($gv->LC_pn_srcregion eq "circle" || $gv->LC_pn_srcregion eq "annulus") {$RSRC1     = $gv->LC_pn_srcregion_R1};
		if ($gv->LC_pn_srcregion eq "circle" || $gv->LC_pn_srcregion eq "annulus") {$RSRC2     = $gv->LC_pn_srcregion_R2};
		if ($gv->LC_pn_srcregion eq "box") {$RSRC1     = $gv->LC_pn_srcregion_XLength};
		if ($gv->LC_pn_srcregion eq "box") {$RSRC2     = $gv->LC_pn_srcregion_YLength};
		$BKG_XC    = $gv->LC_pn_bkgregion_X;
		$BKG_YC    = $gv->LC_pn_bkgregion_Y;
		if ($gv->LC_pn_bkgregion eq "circle" || $gv->LC_pn_bkgregion eq "annulus") {$RBKG1     = $gv->LC_pn_bkgregion_R1};
		if ($gv->LC_pn_bkgregion eq "circle" || $gv->LC_pn_bkgregion eq "annulus") {$RBKG2     = $gv->LC_pn_bkgregion_R2};
		if ($gv->LC_pn_bkgregion eq "box") {$RBKG1     = $gv->LC_pn_bkgregion_XLength};
		if ($gv->LC_pn_bkgregion eq "box") {$RBKG2     = $gv->LC_pn_bkgregion_YLength};
		$T         = $gv->LC_pn_timebinsize;
		$region    = $gv->LC_pn_srcregion;
	
		if (-e $pn_sourcebkgspectrum_file && -e $pn_bkgspectrum_file){
	    	($src_livetime,$src_backscale,$src_areascal)=&event_file_spectralinf($pn_sourcebkgspectrum_file,"pn");
	    	($bkg_livetime,$bkg_backscale,$bkg_areascal)=&event_file_spectralinf($pn_bkgspectrum_file,"pn");
	    	$src_scale = 1.;
	    	$bkg_scale = ($src_backscale/$bkg_backscale);
	    	$gv->LC_pn_areafactor($bkg_scale); 
		} 
		$src_scale = 1.; 
		$bkg_scale = $gv->LC_pn_areafactor;        
		$E1        = $gv->LC_pn_e1_low;            # Low energy limit for 1st light curve
		$E2        = $gv->LC_pn_e1_up;             # High energy limit for 1st light curve 
	
		if ($gv->LC_pn_src_expr eq "") {			
			&produce_LC($pn_gti_cleanevents_file,$region,$SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$BKG_XC,$BKG_YC,$RBKG1,$RBKG2,$E1,$E2,$T,"pn",$pn_sourcelc_e0_file,$pn_bkglc_e0_file,$pn_srcbkgsubtractlc_e0_file,$pn_obsmode);
		} else {
			&produce_user_LC($pn_gti_cleanevents_file,$gv->LC_pn_src_expr,$gv->LC_pn_bkg_expr,$E1,$E2,$T,"pn",$pn_sourcelc_e0_file,$pn_bkglc_e0_file,$pn_srcbkgsubtractlc_e0_file);
		}
    }
    
    if ($m1_lcurve_flag){
		$SRC_XC    = $gv->LC_m1_srcregion_X;
		$SRC_YC    = $gv->LC_m1_srcregion_Y;
		if ($gv->LC_m1_srcregion eq "circle" || $gv->LC_m1_srcregion eq "annulus") {$RSRC1     = $gv->LC_m1_srcregion_R1};
		if ($gv->LC_m1_srcregion eq "circle" || $gv->LC_m1_srcregion eq "annulus") {$RSRC2     = $gv->LC_m1_srcregion_R2};
		if ($gv->LC_m1_srcregion eq "box") {$RSRC1     = $gv->LC_m1_srcregion_XLength};
		if ($gv->LC_m1_srcregion eq "box") {$RSRC2     = $gv->LC_m1_srcregion_YLength};
		$BKG_XC    = $gv->LC_m1_bkgregion_X;
		$BKG_YC    = $gv->LC_m1_bkgregion_Y;
		if ($gv->LC_m1_bkgregion eq "circle" || $gv->LC_m1_bkgregion eq "annulus") {$RBKG1     = $gv->LC_m1_bkgregion_R1};
		if ($gv->LC_m1_bkgregion eq "circle" || $gv->LC_m1_bkgregion eq "annulus") {$RBKG2     = $gv->LC_m1_bkgregion_R2};
		if ($gv->LC_m1_bkgregion eq "box") {$RBKG1     = $gv->LC_m1_bkgregion_XLength};
		if ($gv->LC_m1_bkgregion eq "box") {$RBKG2     = $gv->LC_m1_bkgregion_YLength};
		$T         = $gv->LC_m1_timebinsize;
		$region    = $gv->LC_m1_srcregion;
		if (-e $m1_sourcebkgspectrum_file && -e $m1_bkgspectrum_file){
		    ($src_livetime,$src_backscale,$src_areascal)=&event_file_spectralinf($m1_sourcebkgspectrum_file,"mos1");
		    ($bkg_livetime,$bkg_backscale,$bkg_areascal)=&event_file_spectralinf($m1_bkgspectrum_file,"mos1");
		    $src_scale = 1.;
		    $bkg_scale = ($src_backscale/$bkg_backscale);
		    $gv->LC_m1_areafactor($bkg_scale);
		} 
		$src_scale = 1.;
		$bkg_scale = $gv->LC_m1_areafactor;
		$E1        = $gv->LC_m1_e1_low;             # Low energy limit for 1st light curve
		$E2        = $gv->LC_m1_e1_up;              # High energy limit for 1st light curve
		if ($gv->LC_m1_src_expr eq "") {
			&produce_LC($m1_gti_cleanevents_file,$region,$SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$BKG_XC,$BKG_YC,$RBKG1,$RBKG2,$E1,$E2,$T,"mos1",$m1_sourcelc_e0_file,$m1_bkglc_e0_file,$m1_srcbkgsubtractlc_e0_file,$m1_obsmode);
		} else {
			&produce_user_LC($m1_gti_cleanevents_file,$gv->LC_m1_src_expr,$gv->LC_m1_bkg_expr,$E1,$E2,$T,"mos1",$m1_sourcelc_e0_file,$m1_bkglc_e0_file,$m1_srcbkgsubtractlc_e0_file);
		}
    }    

    if ($m2_lcurve_flag){
		$SRC_XC    = $gv->LC_m2_srcregion_X;
		$SRC_YC    = $gv->LC_m2_srcregion_Y;
		if ($gv->LC_m2_srcregion eq "circle" || $gv->LC_m2_srcregion eq "annulus") {$RSRC1     = $gv->LC_m2_srcregion_R1};
		if ($gv->LC_m2_srcregion eq "circle" || $gv->LC_m2_srcregion eq "annulus") {$RSRC2     = $gv->LC_m2_srcregion_R2};
		if ($gv->LC_m2_srcregion eq "box") {$RSRC1     = $gv->LC_m2_srcregion_XLength};
		if ($gv->LC_m2_srcregion eq "box") {$RSRC2     = $gv->LC_m2_srcregion_YLength};
		$BKG_XC    = $gv->LC_m2_bkgregion_X;
		$BKG_YC    = $gv->LC_m2_bkgregion_Y;
		if ($gv->LC_m2_bkgregion eq "circle" || $gv->LC_m2_bkgregion eq "annulus") {$RBKG1     = $gv->LC_m2_bkgregion_R1};
		if ($gv->LC_m2_bkgregion eq "circle" || $gv->LC_m2_bkgregion eq "annulus") {$RBKG2     = $gv->LC_m2_bkgregion_R2};
		if ($gv->LC_m2_bkgregion eq "box") {$RBKG1     = $gv->LC_m2_bkgregion_XLength};
		if ($gv->LC_m2_bkgregion eq "box") {$RBKG2     = $gv->LC_m2_bkgregion_YLength};
		$T         = $gv->LC_m2_timebinsize;
		$region    = $gv->LC_m2_srcregion;
		if (-e $m2_sourcebkgspectrum_file && -e $m2_bkgspectrum_file){
		    ($src_livetime,$src_backscale,$src_areascal)=&event_file_spectralinf($m2_sourcebkgspectrum_file,"mos2");
		    ($bkg_livetime,$bkg_backscale,$bkg_areascal)=&event_file_spectralinf($m2_bkgspectrum_file,"mos2");
		    $src_scale = 1.;
		    $bkg_scale = ($src_backscale/$bkg_backscale);
		    $gv->LC_m2_areafactor($bkg_scale);
		} 
		$src_scale = 1.; 
		$bkg_scale = $gv->LC_m2_areafactor;
		$E1        = $gv->LC_m2_e1_low;            # Low energy limit for 1st light curve
		$E2        = $gv->LC_m2_e1_up;             # High energy limit for 1st light curve
		if ($gv->LC_m2_src_expr eq "") {
			&produce_LC($m2_gti_cleanevents_file,$region,$SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$BKG_XC,$BKG_YC,$RBKG1,$RBKG2,$E1,$E2,$T,"mos2",$m2_sourcelc_e0_file,$m2_bkglc_e0_file,$m2_srcbkgsubtractlc_e0_file,$m2_obsmode);
		} else {
			&produce_user_LC($m2_gti_cleanevents_file,$gv->LC_m2_src_expr,$gv->LC_m2_bkg_expr,$E1,$E2,$T,"mos2",$m2_sourcelc_e0_file,$m2_bkglc_e0_file,$m2_srcbkgsubtractlc_e0_file);			
		}		
    }

#.. Write Summary
    
    print "\n   \#> Light Curve Analysis Summary :\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Light Curve Analysis Summary :\n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "\n   \#> Light Curve Analysis Summary :\n";}    
    if (!$pn_lcurve_flag){$pn_summ_ana_msg = "FAILED";} else {$pn_summ_ana_msg = "OK";} 
    if (!$m1_lcurve_flag){$m1_summ_ana_msg = "FAILED";} else {$m1_summ_ana_msg = "OK";} 
    if (!$m2_lcurve_flag){$m2_summ_ana_msg = "FAILED";} else {$m2_summ_ana_msg = "OK";} 
    print "      pn : lcurve : $pn_summ_ana_msg \n";
    if ($Log_Flag == 0) {print MYOUTFILE "      pn : lcurve : $pn_summ_ana_msg \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "      pn : lcurve : $pn_summ_ana_msg \n";}  
    print "      mos1 : lcurve : $m1_summ_ana_msg \n";
    if ($Log_Flag == 0) {print MYOUTFILE "      mos1 : lcurve : $m1_summ_ana_msg \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "      mos1 : lcurve : $m1_summ_ana_msg \n";}  
    print "      mos2 : lcurve : $m2_summ_ana_msg \n";
    if ($Log_Flag == 0) {print MYOUTFILE "      mos2 : lcurve : $m2_summ_ana_msg \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "      mos2 : lcurve : $m2_summ_ana_msg \n";}  
      
    if ($Log_Lcur_Flag == 0) {close(LCURVELOG);}

    if ($temp_Log_Spec_Flag == 0) {$Log_Spec_Flag = 0;} # Re-activate Spectral Log flag if active
 
    &error_code(0);
    return;
}


## @method void produce_LC($events_file,$region,$X_Source,$Y_Source,$Source_inRad,$Source_outRad,$X_BKG,$Y_BKG,$BKG_inRad,$BKG_outRad,$E1,$E2,$T_Bin,$instr,$LC_SRC,$LC_BKG,$LC_corrected,$subMode)
# Creates LC.
# @param event_file Event File Name
# @param region Type of Region (circle, annulus, box)
# @param X_Source Source X_coordinate (Image Coordinates)
# @param Y_Source Source Y_coordinate (Image Coordinates)
# @param Source_inRad Source Region Inner Radius (circle,annulus), Source Region X_Box Size (box) (Image Coordinates) 
# @param Source_outRad Source Outer Radius (circle,annulus), Source Region Y_Box Size (box) (Image Coordinates)
# @param X_BKG Background X_coordinate (Image Coordinates)
# @param Y_BKG Background Y_coordinate (Image Coordinates)
# @param BKG_inRad Background Region Inner Radius (circle,annulus), Background Region X_Box Size (box) (Image Coordinates)
# @param BKG_outRad Background Region Outer Radius (circle,annulus), Background Region Y_Box Size (box) (Image Coordinates)
# @param E1 Lower Bound Energy Range (1st range)
# @param E2 Upper Bound Energy Range (1st range)
# @param T_Bin Time Bin (secs)
# @param instr Instrument
# @param LC_SRC Source Light Curve Output File Name
# @param LC_BKG Background Light Curve Output File Name
# @param LC_corrected Light Curve corrected Output File Name
# @subMode Observation mode (IMAGING, TIMING)
# @return Void

#========================================================================
sub produce_LC(){   
# Creates a file from a time series created by evselect
	
    my $event_file    = $_[0];
    my @file_t        = split/\//,$event_file;
    my $file          = $gti_directory.$file_t[$#file_t];
    my $region        = $_[1];
    my $X_Source      = $_[2];
    my $Y_Source      = $_[3];
    my $Source_inRad  = $_[4];
    my $Source_outRad = $_[5];
    my $X_BKG         = $_[6];
    my $Y_BKG         = $_[7];
    my $BKG_inRad     = $_[8];
    my $BKG_outRad    = $_[9];
    my $E1            = $_[10];
    my $E2            = $_[11];
    my $T_Bin         = $_[12];
    my $instr         = $_[13];
    my $src_outfile   = $_[14];
    my $bkg_outfile   = $_[15];
	my $LC_corrected  = $_[16];
	my $subMode		  = $_[17];

    my $source_name                = $gv->SRC_Name;
    my $mode                       = $obsid_instruments{$gv->SRC_OBSID}{$instr}{'mode'};
    my $submode                    = $obsid_instruments{$gv->SRC_OBSID}{$instr}{'submode'};

	&produce_lightcurve_file($E1,$E2,$X_Source,$Y_Source,$Source_inRad,$Source_outRad,$T_Bin,$region,$instr,$event_file,$src_outfile,$subMode);		
	&produce_lightcurve_file($E1,$E2,$X_BKG,$Y_BKG,$BKG_inRad,$BKG_outRad,$T_Bin,$region,$instr,$event_file,$bkg_outfile,$subMode);

#.. epiclccorr: 
    @args = ("epiclccorr","srctslist=$src_outfile","eventlist=$event_file","outset=$LC_corrected","withbkgset=yes","bkgtslist=$bkg_outfile"
	     ,"applyabsolutecorrections=yes");

 	system(@args)== 0 or &error_code(7);
    
    return(1);
}

## @method void produce_user_LC($events_file,$src_expr,$bkg_expr,$E1,$E2,$T_Bin,$instr,$LC_SRC,$LC_BKG,$LC_corrected)
# Creates LC.
# @param event_file Event File Name
# @param source expression
# @param background expression
# @param E1 Lower Bound Energy Range (1st range)
# @param E2 Upper Bound Energy Range (1st range)
# @param T_Bin Time Bin (secs)
# @param instr Instrument
# @param LC_SRC Source Light Curve Output File Name
# @param LC_BKG Background Light Curve Output File Name
# @param LC_corrected Light Curve corrected Output File Name
# @return Void

#========================================================================
sub produce_user_LC(){   
# Creates a file from a time series created by evselect
	
    my $event_file    = $_[0];
    my @file_t        = split/\//,$event_file;
    my $file          = $gti_directory.$file_t[$#file_t];
    my $src_expr      = $_[1];
    my $bkg_expr      = $_[2];
    my $E1            = $_[3];
    my $E2            = $_[4];
    my $T_Bin         = $_[5];
    my $instr         = $_[6];
    my $src_outfile   = $_[7];
    my $bkg_outfile   = $_[8];
	my $LC_corrected  = $_[9];	

    my $source_name                = $gv->SRC_Name;
    my $mode                       = $obsid_instruments{$gv->SRC_OBSID}{$instr}{'mode'};
    my $submode                    = $obsid_instruments{$gv->SRC_OBSID}{$instr}{'submode'};

   
	&produce_user_lightcurve_file($E1,$E2,$src_expr,$T_Bin,$instr,$event_file,$src_outfile);		
	&produce_user_lightcurve_file($E1,$E2,$bkg_expr,$T_Bin,$instr,$event_file,$bkg_outfile);

#.. epiclccorr: 
    @args = ("epiclccorr","srctslist=$src_outfile","eventlist=$event_file","outset=$LC_corrected","withbkgset=yes","bkgtslist=$bkg_outfile"
	     ,"applyabsolutecorrections=yes");

 	system(@args)== 0 or &error_code(7);
    
    return(1);
}

## @method void produce_lightcurve_file($E1,$E2,$XC,$YC,$R1,$R2,$T,$region,$instr,$event_file,$outfile,$subMode)
# Produce a time series given region (circle, annulus or box) and energy range (E1, E2).
# @param E1 Low Bound Energy Range (eV)	   
# @param E2 High Bound Energy Range (eV)	   
# @param XC X-Center of Source Region (Image Pixels)	   
# @param YC Y-Center of Source Region (Image Pixels)	   
# @param R1 Radius/Length of Source Region (Image Pixels)
# @param R2 Radius/Length of Source Region (Image Pixels)
# @param T Time Bin (secs)	   
# @param region	Region Type (circle, annulus, box)   
# @param instr Instrument	   
# @param event_file Event File Name
# @param outfile Output File Name (Rate Set)	    
# @subMode Observation Mode 
# @return Void

#========================================================================
sub produce_lightcurve_file(){
    
# Produce an event file for a given region and energy range

    my $E1      = $_[0];
    my $E2      = $_[1];
    my $XC      = $_[2];
    my $YC      = $_[3];
    my $R1      = $_[4];
    my $R2      = $_[5];
    my $T       = $_[6];
    my $region  = $_[7];
    my $instr   = $_[8];
    my $event_file = $_[9];
    my $outfile    = $_[10];
    my $subMode = $_[11];
	my $xVal;
    my $yVal;
    
    print "       $instr : $region : Energy Range: $E1 - $E2 eV \n";
    print "       $instr : $region : Time Bining : $T sec\n";
    print "       (Event File: $event_file ; Output File: $outfile) \n";
    if ($Log_Flag == 0) {print MYOUTFILE "       $instr : $region : Energy Range: $E1 - $E2 eV \n";}
    if ($Log_Flag == 0) {print MYOUTFILE "       $instr : $region : Time Bining : $T sec\n";}
    if ($Log_Flag == 0) {print MYOUTFILE "       (Event File: $event_file ; Output File: $outfile) \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "       $instr : $region : Energy Range: $E1 - $E2 eV \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "       $instr : $region : Time Bining : $T sec\n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "       (Event File: $event_file ; Output File: $outfile) \n";}  

    if ($instr eq "pn") {$expr="(PI>=$E1) && (PI<=$E2)";}
    if ($instr eq "mos1" or $instr eq "mos2") {$expr="(PI>=$E1) && (PI<=$E2)";}

    if (($instr eq "mos1" or $instr eq "mos2") && (uc($subMode) eq 'TIMING'))
    {    	       		
    	#$areaexpr="((RAWX,TIME) in box($XC,$YC,$R1,$R2,0))";
    	my $X_min = $XC - $R1/2.;
    	my $X_max = $XC + $R1/2.; 
    	$areaexpr="(RAWX in [$X_min:$X_max])";
    } 
    else
    {
	    if ($region eq "circle" || $region eq "annulus") {$areaexpr="((X,Y) in annulus($XC,$YC,$R1,$R2))";}
   		if ($region eq "box") 
   		{
    		if ($subMode eq 'IMAGING' || $subMode eq 'imaging') {
    			$xVal = "X";
    			$yVal = "Y";
    		}
    		else 
    		{
    			$xVal = "RAWX";
    			$yVal = "RAWY";
    		}
    	
    		$areaexpr="(($xVal,$yVal) in box($XC,$YC,$R1,$R2,0))";
   		}
    }
    @args = ("evselect table=$event_file:EVENTS expression=\"$expr \&\& $areaexpr\" rateset=$outfile timebinsize=$T withrateset=yes maketimecolumn=yes makeratecolumn=yes");

    system(@args)== 0 or &error_code(7);
    return(1);
}

## @method void produce_user_lightcurve_file($E1,$E2,$XC,$YC,$R1,$R2,$T,$region,$instr,$event_file,$outfile,$subMode)
# Produce a time series given region (circle, annulus or box) and energy range (E1, E2).
# @param E1 Low Bound Energy Range (eV)	   
# @param E2 High Bound Energy Range (eV)	   
# @param expression	   	   
# @param R1 Radius/Length of Source Region (Image Pixels)
# @param R2 Radius/Length of Source Region (Image Pixels)
# @param T Time Bin (secs)	   
# @param region	Region Type (circle, annulus, box)   
# @param instr Instrument	   
# @param event_file Event File Name
# @param outfile Output File Name (Rate Set)	    
# @subMode Observation Mode 
# @return Void

#========================================================================
sub produce_user_lightcurve_file(){
    
# Produce an event file for a given region and energy range

    my $E1      = $_[0];
    my $E2      = $_[1];
    my $expression  = $_[2];    
    my $T       = $_[3];
    my $instr   = $_[4];
    my $event_file = $_[5];
    my $outfile    = $_[6];
    
    print "       $instr : $expression : Energy Range: $E1 - $E2 eV \n";    
    print "       $instr : Time Bining : $T sec\n";
    print "       (Event File: $event_file ; Output File: $outfile) \n";
    if ($Log_Flag == 0) {print MYOUTFILE "       $instr : $expression : Energy Range: $E1 - $E2 eV \n";}
    if ($Log_Flag == 0) {print MYOUTFILE "       $instr : Time Bining : $T sec\n";}
    if ($Log_Flag == 0) {print MYOUTFILE "       (Event File: $event_file ; Output File: $outfile) \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "       $instr :  $expression : Energy Range: $E1 - $E2 eV \n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "       $instr :  Time Bining : $T sec\n";}
    if ($Log_Lcur_Flag == 0) {print LCURVELOG "       (Event File: $event_file ; Output File: $outfile) \n";}  

    if ($instr eq "pn") {$expr="(PI>=$E1) && (PI<=$E2)";}
    if ($instr eq "mos1" or $instr eq "mos2") {$expr="(PI>=$E1) && (PI<=$E2)";}

    @args = ("evselect table=$event_file:EVENTS expression=\"$expr \&\& ($expression)\" rateset=$outfile timebinsize=$T withrateset=yes maketimecolumn=yes makeratecolumn=yes");

    system(@args)== 0 or &error_code(7);
    return(1);
}


1;
